"""Composable helpers shared by the module scaffolder."""

from __future__ import annotations

from .constants import (
    CLI_ROOT,
    MODULES_ROOT,
    PACKAGE_ROOT,
    REPO_ROOT,
    SRC_ROOT,
    UTILS_ROOT,
)
from .identifiers import derive_identifiers
from .result import ModuleScaffoldResult

__all__ = [
    "derive_identifiers",
    "ModuleScaffoldResult",
    "CLI_ROOT",
    "MODULES_ROOT",
    "PACKAGE_ROOT",
    "REPO_ROOT",
    "SRC_ROOT",
    "UTILS_ROOT",
]



"""Override contracts for the CORS module."""

from __future__ import annotations

import json
import os
from dataclasses import dataclass
from pathlib import Path
from typing import Any, Mapping, Optional, Tuple

from core.services.override_contracts import ConfigurableOverrideMixin

TRUTHY_VALUES = {"1", "true", "yes", "on"}
FALSY_VALUES = {"0", "false", "no", "off"}


@dataclass(frozen=True)
class CorsOverrideState:
    """Snapshot of environment-driven overrides for CORS defaults."""

    enabled: Optional[bool] = None
    allow_origins: Tuple[str, ...] = ()
    allow_credentials: Optional[bool] = None
    allow_methods: Tuple[str, ...] = ()
    allow_headers: Tuple[str, ...] = ()
    expose_headers: Tuple[str, ...] = ()
    max_age: Optional[int] = None
    log_level: Optional[str] = None
    metadata: Optional[dict[str, Any]] = None


def _get_env(name: str) -> Optional[str]:
    value = os.getenv(name)
    if value is None:
        return None
    stripped = value.strip()
    return stripped or None


def _parse_bool(name: str) -> Optional[bool]:
    raw = _get_env(name)
    if raw is None:
        return None
    lowered = raw.lower()
    if lowered in TRUTHY_VALUES:
        return True
    if lowered in FALSY_VALUES:
        return False
    return None


def _parse_int(name: str) -> Optional[int]:
    raw = _get_env(name)
    if raw is None:
        return None
    try:
        return int(raw)
    except ValueError:
        return None


def _parse_sequence(name: str) -> Tuple[str, ...]:
    raw = _get_env(name)
    if not raw:
        return ()
    if raw.startswith("["):
        try:
            resolved = json.loads(raw)
        except json.JSONDecodeError:
            resolved = None
        if isinstance(resolved, (list, tuple)):
            return tuple(str(item).strip() for item in resolved if str(item).strip())
    return tuple(part.strip() for part in raw.split(",") if part.strip())


def _parse_mapping(name: str) -> Optional[dict[str, Any]]:
    raw = _get_env(name)
    if not raw:
        return None
    try:
        payload = json.loads(raw)
    except json.JSONDecodeError:
        return None
    if isinstance(payload, dict):
        return {str(key): value for key, value in payload.items()}
    return None


def resolve_override_state(_: Path) -> CorsOverrideState:
    prefix = "RAPIDKIT_CORS"
    return CorsOverrideState(
        enabled=_parse_bool(f"{prefix}_ENABLED"),
        allow_origins=_parse_sequence(f"{prefix}_ALLOW_ORIGINS"),
        allow_credentials=_parse_bool(f"{prefix}_ALLOW_CREDENTIALS"),
        allow_methods=_parse_sequence(f"{prefix}_ALLOW_METHODS"),
        allow_headers=_parse_sequence(f"{prefix}_ALLOW_HEADERS"),
        expose_headers=_parse_sequence(f"{prefix}_EXPOSE_HEADERS"),
        max_age=_parse_int(f"{prefix}_MAX_AGE"),
        log_level=_get_env(f"{prefix}_LOG_LEVEL"),
        metadata=_parse_mapping(f"{prefix}_METADATA"),
    )


def _mutate_defaults(defaults: Mapping[str, Any], state: CorsOverrideState) -> dict[str, Any]:
    mutated = dict(defaults)
    if state.enabled is not None:
        mutated["enabled"] = state.enabled
    if state.allow_origins:
        mutated["allow_origins"] = list(state.allow_origins)
    if state.allow_credentials is not None:
        mutated["allow_credentials"] = state.allow_credentials
    if state.allow_methods:
        mutated["allow_methods"] = list(state.allow_methods)
    if state.allow_headers:
        mutated["allow_headers"] = list(state.allow_headers)
    if state.expose_headers:
        mutated["expose_headers"] = list(state.expose_headers)
    if state.max_age is not None:
        mutated["max_age"] = max(state.max_age, 0)
    if state.log_level:
        mutated["log_level"] = state.log_level
    if state.metadata:
        metadata = dict(mutated.get("metadata", {}))
        metadata.update(state.metadata)
        mutated["metadata"] = metadata
    return mutated


class CorsOverrides(ConfigurableOverrideMixin):
    """Extend or customize generated behaviour for the CORS module."""

    def __init__(self, module_root: Path | None = None) -> None:
        self.module_root = module_root or Path(__file__).resolve().parent
        self.state = resolve_override_state(self.module_root)
        super().__init__()

    def apply_base_context(self, context: Mapping[str, Any]) -> dict[str, Any]:
        mutated = dict(context)
        defaults = mutated.get("cors_defaults", {})
        if not isinstance(defaults, Mapping):
            defaults = {}
        mutated_defaults = _mutate_defaults(defaults, self.state)
        mutated["cors_defaults"] = mutated_defaults
        mutated.setdefault("override_metadata", {})
        mutated["override_metadata"].update(
            {
                "cors_env_overrides": True,
                "cors_sequence_overrides": bool(
                    self.state.allow_origins
                    or self.state.allow_methods
                    or self.state.allow_headers
                    or self.state.expose_headers
                ),
                "cors_metadata_overrides": bool(self.state.metadata),
                "cors_log_level_override": bool(self.state.log_level),
            }
        )
        mutated["cors_override_state"] = self.state
        return mutated

    def apply_variant_context_pre(
        self,
        context: Mapping[str, Any],
        *,
        variant_name: str,
    ) -> dict[str, Any]:
        enriched = dict(context)
        enriched.setdefault("framework", variant_name)
        enriched.setdefault("cors_override_state", self.state)
        return enriched

    def apply_variant_context_post(
        self,
        context: Mapping[str, Any],
        *,
        variant_name: str,
    ) -> dict[str, Any]:
        _ = variant_name  # Deliberately unused yet retained for future parity hooks.
        return dict(context)

    def post_variant_generation(
        self,
        *,
        variant_name: str,
        target_dir: Path,
        enriched_context: Mapping[str, Any],
    ) -> None:
        _ = (variant_name, target_dir, enriched_context)
        # Reserved for future hooks (e.g., documentation injection).


__all__ = ["CorsOverrides", "CorsOverrideState", "resolve_override_state"]



"""Override contracts for Observability Core."""

from __future__ import annotations

import json
import os
from dataclasses import dataclass
from pathlib import Path
from typing import Any, Mapping, Optional

from core.services.override_contracts import ConfigurableOverrideMixin

DEFAULTS_KEY = "observability_defaults"

TRUTHY_VALUES = {"1", "true", "yes", "on"}
FALSY_VALUES = {"0", "false", "no", "off"}


@dataclass(frozen=True)
class ObservabilityOverrideState:
    """Snapshot of environment-driven overrides for observability defaults."""

    service_name: Optional[str] = None
    environment: Optional[str] = None
    resource_attributes: Optional[dict[str, str]] = None

    metrics_enabled: Optional[bool] = None
    metrics_exporter: Optional[str] = None
    metrics_endpoint: Optional[str] = None
    metrics_namespace: Optional[str] = None
    metrics_labels: Optional[dict[str, str]] = None
    metrics_buckets: Optional[tuple[float, ...]] = None
    metrics_retention: Optional[int] = None
    metrics_process_metrics: Optional[bool] = None

    tracing_enabled: Optional[bool] = None
    tracing_exporter: Optional[str] = None
    tracing_endpoint: Optional[str] = None
    tracing_sample_ratio: Optional[float] = None
    tracing_include_headers: Optional[bool] = None

    logging_level: Optional[str] = None
    logging_structured: Optional[bool] = None
    logging_include_trace_ids: Optional[bool] = None

    events_buffer_size: Optional[int] = None
    events_flush_interval: Optional[int] = None
    events_audit_enabled: Optional[bool] = None
    retry_attempts: Optional[int] = None


def _get_env(name: str) -> Optional[str]:
    value = os.getenv(name)
    if value is None:
        return None
    stripped = value.strip()
    return stripped or None


def _parse_bool(name: str) -> Optional[bool]:
    value = _get_env(name)
    if value is None:
        return None
    lowered = value.lower()
    if lowered in TRUTHY_VALUES:
        return True
    if lowered in FALSY_VALUES:
        return False
    return None


def _parse_int(name: str) -> Optional[int]:
    value = _get_env(name)
    if value is None:
        return None
    try:
        return int(value)
    except ValueError:
        return None


def _parse_float(name: str) -> Optional[float]:
    value = _get_env(name)
    if value is None:
        return None
    try:
        return float(value)
    except ValueError:
        return None


def _parse_mapping(name: str) -> Optional[dict[str, str]]:
    value = _get_env(name)
    if value is None:
        return None
    try:
        parsed = json.loads(value)
    except json.JSONDecodeError:
        parsed = value

    if isinstance(parsed, Mapping):
        return {str(key): str(val) for key, val in parsed.items()}

    mapping: dict[str, str] = {}
    if isinstance(parsed, str):
        for segment in parsed.split(","):
            if "=" not in segment:
                continue
            key, raw_value = segment.split("=", 1)
            key = key.strip()
            if not key:
                continue
            mapping[key] = raw_value.strip()
    return mapping or None


def _parse_float_list(name: str) -> Optional[tuple[float, ...]]:
    value = _get_env(name)
    if value is None:
        return None

    try:
        parsed = json.loads(value)
    except json.JSONDecodeError:
        parsed = value

    numbers: list[float] = []
    if isinstance(parsed, (list, tuple)):
        for element in parsed:
            try:
                numbers.append(float(element))
            except (TypeError, ValueError):
                continue
    elif isinstance(parsed, str):
        for raw_chunk in parsed.split(","):
            chunk = raw_chunk.strip()
            if not chunk:
                continue
            try:
                numbers.append(float(chunk))
            except ValueError:
                continue

    deduped = list(dict.fromkeys(numbers))
    return tuple(deduped) or None


def resolve_override_state(
    _module_root: Path | None = None,
) -> ObservabilityOverrideState:
    """Resolve overrides from environment variables."""

    return ObservabilityOverrideState(
        service_name=_get_env("RAPIDKIT_OBSERVABILITY_SERVICE_NAME"),
        environment=_get_env("RAPIDKIT_OBSERVABILITY_ENVIRONMENT"),
        resource_attributes=_parse_mapping("RAPIDKIT_OBSERVABILITY_RESOURCE_ATTRS"),
        metrics_enabled=_parse_bool("RAPIDKIT_OBSERVABILITY_METRICS_ENABLED"),
        metrics_exporter=_get_env("RAPIDKIT_OBSERVABILITY_METRICS_EXPORTER"),
        metrics_endpoint=_get_env("RAPIDKIT_OBSERVABILITY_METRICS_ENDPOINT"),
        metrics_namespace=_get_env("RAPIDKIT_OBSERVABILITY_METRICS_NAMESPACE"),
        metrics_labels=_parse_mapping("RAPIDKIT_OBSERVABILITY_METRICS_LABELS"),
        metrics_buckets=_parse_float_list("RAPIDKIT_OBSERVABILITY_METRICS_BUCKETS"),
        metrics_retention=_parse_int("RAPIDKIT_OBSERVABILITY_METRICS_RETENTION"),
        metrics_process_metrics=_parse_bool("RAPIDKIT_OBSERVABILITY_METRICS_PROCESS"),
        tracing_enabled=_parse_bool("RAPIDKIT_OBSERVABILITY_TRACING_ENABLED"),
        tracing_exporter=_get_env("RAPIDKIT_OBSERVABILITY_TRACING_EXPORTER"),
        tracing_endpoint=_get_env("RAPIDKIT_OBSERVABILITY_TRACING_ENDPOINT"),
        tracing_sample_ratio=_parse_float("RAPIDKIT_OBSERVABILITY_TRACING_SAMPLE_RATIO"),
        tracing_include_headers=_parse_bool("RAPIDKIT_OBSERVABILITY_TRACING_HEADERS"),
        logging_level=_get_env("RAPIDKIT_OBSERVABILITY_LOG_LEVEL"),
        logging_structured=_parse_bool("RAPIDKIT_OBSERVABILITY_STRUCTURED_LOGGING"),
        logging_include_trace_ids=_parse_bool("RAPIDKIT_OBSERVABILITY_INCLUDE_TRACE_IDS"),
        events_buffer_size=_parse_int("RAPIDKIT_OBSERVABILITY_EVENTS_BUFFER"),
        events_flush_interval=_parse_int("RAPIDKIT_OBSERVABILITY_EVENTS_FLUSH_INTERVAL"),
        events_audit_enabled=_parse_bool("RAPIDKIT_OBSERVABILITY_EVENTS_AUDIT"),
        retry_attempts=_parse_int("RAPIDKIT_OBSERVABILITY_RETRY_ATTEMPTS"),
    )


def _merge_nested(mapping: Mapping[str, Any] | None) -> dict[str, Any]:
    if not isinstance(mapping, Mapping):
        return {}
    return {
        key: (dict(value) if isinstance(value, Mapping) else value)
        for key, value in mapping.items()
    }


def _mutate_defaults(
    defaults: Mapping[str, Any], state: ObservabilityOverrideState
) -> dict[str, Any]:
    mutated = _merge_nested(defaults)

    if state.service_name:
        mutated["service_name"] = state.service_name
    if state.environment:
        mutated["environment"] = state.environment

    resource_attributes = _merge_nested(mutated.get("resource_attributes", {}))
    if state.resource_attributes:
        resource_attributes.update(state.resource_attributes)
    mutated["resource_attributes"] = resource_attributes

    metrics = _merge_nested(mutated.get("metrics", {}))
    if state.metrics_enabled is not None:
        metrics["enabled"] = state.metrics_enabled
    if state.metrics_exporter:
        metrics["exporter"] = state.metrics_exporter
    if state.metrics_endpoint:
        metrics["endpoint"] = state.metrics_endpoint
    if state.metrics_namespace:
        metrics["namespace"] = state.metrics_namespace
    if state.metrics_labels:
        default_labels = _merge_nested(metrics.get("default_labels", {}))
        default_labels.update(state.metrics_labels)
        metrics["default_labels"] = default_labels
    if state.metrics_buckets is not None:
        metrics["buckets"] = list(state.metrics_buckets)
    if state.metrics_retention is not None:
        metrics["retention_seconds"] = max(state.metrics_retention, 0)
    if state.metrics_process_metrics is not None:
        metrics["register_process_metrics"] = state.metrics_process_metrics
    mutated["metrics"] = metrics

    tracing = _merge_nested(mutated.get("tracing", {}))
    if state.tracing_enabled is not None:
        tracing["enabled"] = state.tracing_enabled
    if state.tracing_exporter:
        tracing["exporter"] = state.tracing_exporter
    if state.tracing_endpoint:
        tracing["endpoint"] = state.tracing_endpoint
    if state.tracing_sample_ratio is not None:
        tracing["sample_ratio"] = max(0.0, min(state.tracing_sample_ratio, 1.0))
    if state.tracing_include_headers is not None:
        tracing["include_headers"] = state.tracing_include_headers
    mutated["tracing"] = tracing

    events = _merge_nested(mutated.get("events", {}))
    if state.events_buffer_size is not None:
        events["buffer_size"] = max(state.events_buffer_size, 0)
    if state.events_flush_interval is not None:
        events["flush_interval_seconds"] = max(state.events_flush_interval, 0)
    if state.events_audit_enabled is not None:
        events["audit_enabled"] = state.events_audit_enabled
    mutated["events"] = events

    if state.logging_level:
        mutated["log_level"] = state.logging_level
    if state.logging_structured is not None:
        mutated["structured_logging"] = state.logging_structured
    if state.logging_include_trace_ids is not None:
        mutated["include_trace_ids"] = state.logging_include_trace_ids
    if state.retry_attempts is not None:
        mutated["retry_attempts"] = max(state.retry_attempts, 0)

    return mutated


class ObservabilityCoreOverrides(ConfigurableOverrideMixin):
    """Extend or customize generated behaviour for Observability Core."""

    def __init__(self, module_root: Path | None = None) -> None:
        self.module_root = module_root or Path(__file__).resolve().parent
        self.state = resolve_override_state(self.module_root)
        super().__init__()

    def apply_base_context(self, context: Mapping[str, Any]) -> dict[str, Any]:
        mutated = dict(context)
        defaults = context.get(DEFAULTS_KEY, {})
        mutated[DEFAULTS_KEY] = _mutate_defaults(defaults, self.state)
        return mutated

    def apply_variant_context_pre(
        self,
        context: Mapping[str, Any],
        *,
        variant_name: str,
    ) -> dict[str, Any]:  # noqa: ARG002
        return self.apply_base_context(context)

    def apply_variant_context_post(
        self,
        context: Mapping[str, Any],
        *,
        variant_name: str,
    ) -> dict[str, Any]:  # noqa: ARG002
        return self.apply_base_context(context)

    def post_variant_generation(
        self,
        *,
        variant_name: str,
        target_dir: Path,
        enriched_context: Mapping[str, Any],
    ) -> None:  # noqa: ARG002
        # No post-processing hooks yet. Placeholder for future exporters.
        return None


__all__ = [
    "ObservabilityCoreOverrides",
    "ObservabilityOverrideState",
    "resolve_override_state",
    "DEFAULTS_KEY",
]



# pyright: reportMissingImports=false

"""NestJS framework plugin for settings module code generation."""

from __future__ import annotations

import json
import logging
import shutil
import subprocess  # nosec
from pathlib import Path
from typing import Any, Dict, List, Mapping

from modules.shared.frameworks import FrameworkPlugin
from modules.shared.utils.health_specs import build_standard_health_spec

MODULE_ROOT = Path(__file__).resolve().parents[1]
HEALTH_SHIM_SPEC = build_standard_health_spec(MODULE_ROOT)

# Minimum Node.js version required for NestJS
MIN_NODE_VERSION = 16

logger = logging.getLogger(__name__)  # pylint: disable=E1101

_DEFAULT_TSCONFIG: Dict[str, Any] = {
    "compilerOptions": {
        "module": "commonjs",
        "declaration": True,
        "removeComments": True,
        "emitDecoratorMetadata": True,
        "experimentalDecorators": True,
        "allowSyntheticDefaultImports": True,
        "target": "ES2020",
        "sourceMap": True,
        "outDir": "./dist",
        "baseUrl": "./",
        "incremental": True,
        "skipLibCheck": True,
        "strictNullChecks": False,
        "noImplicitAny": False,
        "strictBindCallApply": False,
        "forceConsistentCasingInFileNames": False,
        "noFallthroughCasesInSwitch": False,
    }
}

_DEFAULT_PACKAGE_JSON: Dict[str, Any] = {
    "name": "nestjs-app",
    "version": "1.0.0",
    "description": "A NestJS application with settings management",
    "author": "",
    "private": True,
    "license": "UNLICENSED",
    "scripts": {
        "build": "nest build",
        "format": 'prettier --write "src/**/*.ts" "test/**/*.ts"',
        "start": "nest start",
        "start:dev": "nest start --watch",
        "start:debug": "nest start --debug --watch",
        "start:prod": "node dist/main",
        "lint": 'eslint "{src,apps,libs,test}/**/*.ts" --fix',
        "test": "jest",
        "test:watch": "jest --watch",
        "test:cov": "jest --coverage",
        "test:debug": "node --inspect-brk -r tsconfig-paths/register -r ts-node/register node_modules/.bin/jest --runInBand",
        "test:e2e": "jest --config ./test/jest-e2e.json",
    },
    "dependencies": {
        "@nestjs/common": "^11.1.6",
        "@nestjs/core": "^11.1.6",
        "@nestjs/platform-express": "^11.1.6",
        "@nestjs/config": "^4.0.2",
        "reflect-metadata": "^0.1.13",
        "rxjs": "^7.8.1",
        "class-validator": "^0.14.0",
        "class-transformer": "^0.5.1",
    },
    "devDependencies": {
        "@nestjs/cli": "^11.0.10",
        "@nestjs/schematics": "^11.0.10",
        "@nestjs/testing": "^11.1.6",
        "@types/express": "^4.17.17",
        "@types/node": "^20.0.0",
        "@types/jest": "^29.5.0",
        "@types/supertest": "^2.0.12",
        "@typescript-eslint/eslint-plugin": "^6.0.0",
        "@typescript-eslint/parser": "^6.0.0",
        "eslint": "^8.42.0",
        "eslint-config-prettier": "^8.8.0",
        "eslint-plugin-prettier": "^4.2.1",
        "jest": "^29.5.0",
        "prettier": "^2.8.8",
        "source-map-support": "^0.5.21",
        "supertest": "^6.3.3",
        "ts-jest": "^29.1.0",
        "ts-loader": "^9.4.3",
        "ts-node": "^10.9.1",
        "tsconfig-paths": "^4.2.1",
        "typescript": "^5.1.3",
    },
    "jest": {
        "moduleFileExtensions": ["js", "json", "ts"],
        "rootDir": "src",
        "testRegex": ".*\\.spec\\.ts$",
        "transform": {"^.+\\.(t|j)s$": "ts-jest"},
        "collectCoverageFrom": ["**/*.(t|j)s"],
        "coverageDirectory": "../coverage",
        "testEnvironment": "node",
    },
}


class NestJSPlugin(FrameworkPlugin):
    """Plugin for generating NestJS-specific settings code."""

    @property
    def name(self) -> str:
        return "nestjs"

    @property
    def language(self) -> str:
        return "typescript"

    @property
    def display_name(self) -> str:
        return "NestJS"

    def get_template_mappings(self) -> Dict[str, str]:
        """Map logical template names to NestJS-specific template files."""
        return {
            "configuration": "templates/variants/nestjs/configuration.ts.j2",
            "settings_service": "templates/variants/nestjs/settings.service.ts.j2",
            "health": "templates/vendor/nestjs/settings.health.ts.j2",
            "settings_controller": "templates/variants/nestjs/settings.controller.ts.j2",
            "settings_module": "templates/variants/nestjs/settings.module.ts.j2",
            "metadata": "templates/variants/nestjs/settings.metadata.ts.j2",
            "integration_tests": "templates/tests/integration/settings.integration.spec.ts.j2",
            "e2e_tests": "templates/variants/nestjs/tests/settings.e2e-spec.ts",
        }

    def get_output_paths(self) -> Dict[str, str]:
        """Map logical names to NestJS output file paths."""
        return {
            "configuration": "src/modules/free/essentials/settings/configuration.ts",
            "settings_service": "src/modules/free/essentials/settings/settings.service.ts",
            "settings_controller": "src/modules/free/essentials/settings/settings.controller.ts",
            "settings_module": "src/modules/free/essentials/settings/settings.module.ts",
            "health": "src/health/settings.health.ts",
            "metadata": "src/modules/free/essentials/settings/settings.metadata.ts",
            "integration_tests": "tests/modules/integration/essentials/settings/settings.integration.spec.ts",
            "e2e_tests": "tests/modules/e2e/free/essentials/settings/settings.e2e-spec.ts",
        }

    def get_context_enrichments(self, base_context: Mapping[str, Any]) -> Dict[str, Any]:
        """Add NestJS-specific context variables."""
        context = dict(base_context)
        module_name = str(
            context.get("module_name") or context.get("rapidkit_vendor_module") or "settings"
        )
        module_segment = module_name.split("/")[-1]

        context.setdefault("module_slug", module_segment)
        context.setdefault("module_kebab", module_segment.replace("_", "-"))
        context.setdefault("module_basename", module_segment)

        context.update(
            framework="nestjs",
            framework_display_name="NestJS",
            language="typescript",
            import_statement="import { Module } from '@nestjs/common'",
            framework_features={
                "dependency_injection": True,
                "decorators": True,
                "modules": True,
                "middleware_support": True,
                "guards_interceptors": True,
            },
            config_decorators={
                "injectable": "@Injectable()",
                "config_service": "@Inject(ConfigService)",
                "module": "@Module({})",
                "global": "@Global()",
            },
            nestjs_imports=[
                "ConfigModule",
                "ConfigService",
                "ConfigType",
            ],
            vendor_configuration_relative="nestjs/configuration.js",
        )
        return context

    def validate_requirements(self) -> List[str]:
        """Validate NestJS-specific requirements."""
        errors = []

        # Check if Node.js is available
        node_bin = shutil.which("node")
        npm_bin = shutil.which("npm")

        if not node_bin:
            errors.append("Node.js is required for NestJS development")
        else:
            # Check Node.js version
            try:
                result = subprocess.run(  # nosec
                    [node_bin, "--version"],
                    capture_output=True,
                    text=True,
                    timeout=5,
                    check=False,  # We check returncode manually
                )
                if result.returncode == 0:
                    version = result.stdout.strip().lstrip("v")
                    major_version = int(version.split(".")[0])
                    if major_version < MIN_NODE_VERSION:
                        errors.append(
                            f"Node.js version {version} is too old. NestJS requires Node.js 16+"
                        )
                else:
                    errors.append("Failed to determine Node.js version")
            except (subprocess.TimeoutExpired, subprocess.SubprocessError):
                errors.append("Failed to check Node.js version")

        if not npm_bin:
            errors.append("npm is required for NestJS package management")

        # Check if TypeScript is available (optional check)
        tsc_bin = shutil.which("tsc") or shutil.which("npx")
        if not tsc_bin:
            errors.append("TypeScript compiler not found - install with: npm install -g typescript")

        return errors

    def get_dependencies(self) -> List[str]:
        """Get NestJS-specific dependencies."""
        return [
            "@nestjs/core",
            "@nestjs/common",
            "@nestjs/config",
            "@nestjs/platform-express",
            "class-validator",
            "class-transformer",
            "reflect-metadata",
            "rxjs",
        ]

    def get_dev_dependencies(self) -> List[str]:
        """Get development dependencies for NestJS."""
        return [
            "@types/node",
            "@types/express",
            "@typescript-eslint/eslint-plugin",
            "@typescript-eslint/parser",
            "eslint",
            "jest",
            "@types/jest",
            "@types/supertest",
            "supertest",
            "ts-jest",
            "typescript",
        ]

    def pre_generation_hook(self, output_dir: Path) -> None:
        """Prepare NestJS-specific setup before generation."""

        base = output_dir / "src" / "modules" / "free" / "essentials" / "settings"
        base.mkdir(parents=True, exist_ok=True)
        (output_dir / "tests" / "modules" / "integration" / "essentials" / "settings").mkdir(
            parents=True, exist_ok=True
        )

        tsconfig_path = output_dir / "tsconfig.json"
        package_path = output_dir / "package.json"

        self._ensure_tsconfig(tsconfig_path)
        self._ensure_package_json(package_path)

        (output_dir / "src" / "health").mkdir(parents=True, exist_ok=True)

    def post_generation_hook(self, output_dir: Path) -> None:
        """Perform NestJS-specific cleanup after generation."""
        # Downstream projects can extend this to run formatters if desired.
        _ = output_dir

    def _ensure_tsconfig(self, tsconfig_path: Path) -> None:
        if tsconfig_path.exists():
            try:
                current = json.loads(tsconfig_path.read_text())
            except json.JSONDecodeError:
                logger.warning(
                    "Existing tsconfig.json is not valid JSON; skipping automatic updates"
                )
                return

            compiler_options = current.setdefault("compilerOptions", {})
            updated = False

            for key, value in _DEFAULT_TSCONFIG["compilerOptions"].items():
                if key not in compiler_options:
                    compiler_options[key] = value
                    updated = True

            if updated:
                tsconfig_path.write_text(json.dumps(current, indent=2) + "\n")
                logger.info("Updated tsconfig.json with missing compiler options")
            return

        tsconfig_path.parent.mkdir(parents=True, exist_ok=True)
        tsconfig_path.write_text(json.dumps(_DEFAULT_TSCONFIG, indent=2) + "\n")
        logger.info("Created tsconfig.json scaffold for NestJS settings module")

    def _ensure_package_json(self, package_path: Path) -> None:
        if package_path.exists():
            try:
                current = json.loads(package_path.read_text())
            except json.JSONDecodeError:
                logger.warning(
                    "Existing package.json is not valid JSON; skipping automatic updates"
                )
                return

            updated = False

            for section in ("dependencies", "devDependencies", "scripts"):
                default_section = _DEFAULT_PACKAGE_JSON.get(section, {})
                if not isinstance(default_section, dict):
                    continue

                target_section = current.setdefault(section, {})
                if not isinstance(target_section, dict):
                    logger.warning(
                        "package.json section '%s' is not a mapping; skipping merge", section
                    )
                    continue

                for key, value in default_section.items():
                    if key not in target_section:
                        target_section[key] = value
                        updated = True

            for root_key in ("name", "version", "description", "private", "license"):
                if root_key not in current:
                    current[root_key] = _DEFAULT_PACKAGE_JSON[root_key]
                    updated = True

            if updated:
                package_path.write_text(json.dumps(current, indent=2) + "\n")
                logger.info("Augmented existing package.json with NestJS defaults")
            return

        package_path.write_text(json.dumps(_DEFAULT_PACKAGE_JSON, indent=2) + "\n")
        logger.info("Created package.json scaffold for NestJS settings module")

    def get_documentation_urls(self) -> Dict[str, str]:
        """Get NestJS documentation URLs."""
        return {
            "framework_docs": "https://docs.nestjs.com/",
            "config_docs": "https://docs.nestjs.com/techniques/configuration",
            "cli_docs": "https://docs.nestjs.com/cli/overview",
            "typescript_setup": "https://docs.nestjs.com/recipes/swc",
        }

    def get_example_configurations(self) -> Dict[str, Any]:
        """Get example NestJS configurations."""
        return {
            "app": {
                "name": "My NestJS App",
                "version": "1.0.0",
                "description": "A NestJS application with settings management",
                "port": 3000,
            },
            "database": {
                "type": "postgres",
                "host": "localhost",
                "port": 5432,
                "username": "postgres",
                "password": "password",
                "database": "mydb",
                "synchronize": True,
            },
            "jwt": {
                "secret": "your-secret-key",
                "expiresIn": "24h",
            },
            "cors": {
                "origin": ["http://localhost:3000", "http://localhost:4200"],
                "credentials": True,
            },
        }


class NestJSStandardPlugin(NestJSPlugin):
    """Alias plugin used by the nestjs.standard kit profile."""

    @property
    def name(self) -> str:  # noqa: D401
        return "nestjs.standard"

    @property
    def display_name(self) -> str:
        return "NestJS (standard kit)"



from typing import Optional

_STATUS_MISSING = "missing"
_STATUS_NEW_TEMPLATE = "new_template"
_STATUS_UNKNOWN = "unknown"
_STATUS_UNTRACKED = "untracked_existing"
_STATUS_CLEAN = "clean"
_STATUS_TEMPLATE_UPDATED = "template_updated"
_STATUS_LOCALLY_MODIFIED = "locally_modified"
_STATUS_MANUALLY_UPDATED = "manually_updated"
_STATUS_DIVERGED = "diverged"


def classify_file_status(
    dst_exists: bool,
    registry_hash: Optional[str],
    current_hash: Optional[str],
    regenerated_hash: Optional[str],
) -> str:
    """Classify file status comparing registry, current, and regenerated template hashes.

    Returns one of predefined status constants.
    """
    status = _STATUS_UNKNOWN
    if not dst_exists:
        if registry_hash:
            status = _STATUS_MISSING
        else:
            status = _STATUS_NEW_TEMPLATE if regenerated_hash else _STATUS_UNKNOWN
    elif not registry_hash:
        # Legacy/initial projects may not have a registry yet. If the file exists
        # and there's no registry entry we treat it as untracked by default.
        # If we can regenerate a template and compare to the current file, we
        # only mark it as locally modified when different; when equal we still
        # consider it an untracked existing file (registry absent).
        if dst_exists and regenerated_hash and current_hash:
            status = (
                _STATUS_UNTRACKED if regenerated_hash == current_hash else _STATUS_LOCALLY_MODIFIED
            )
        else:
            status = _STATUS_UNTRACKED
    else:
        hashes_equal = current_hash == registry_hash
        if not regenerated_hash:
            status = _STATUS_CLEAN if hashes_equal else _STATUS_LOCALLY_MODIFIED
        else:
            template_equal_registry = regenerated_hash == registry_hash
            template_equal_current = regenerated_hash == current_hash
            if hashes_equal:
                status = _STATUS_CLEAN if template_equal_registry else _STATUS_TEMPLATE_UPDATED
            elif template_equal_registry:
                status = _STATUS_LOCALLY_MODIFIED
            elif template_equal_current:
                status = _STATUS_MANUALLY_UPDATED
            else:
                status = _STATUS_DIVERGED
    return status



# src / cli / utils / filesystem.py
from pathlib import Path
from typing import List, Optional, Union


def find_main_py(project_root: Path, base_module: str = "") -> Optional[Path]:
    base_candidates: List[str] = ["main.py", "app/main.py", "src/main.py"]
    if base_module:
        base_candidates.append(f"{base_module}/main.py")
    for rel in base_candidates:
        candidate_path = project_root / rel
        if candidate_path.exists():
            return candidate_path
    return None


def create_file(path: Path, content: str = "") -> None:
    path.parent.mkdir(parents=True, exist_ok=True)
    path.write_text(content, encoding="utf-8")


def find_project_root(target_project: Optional[str] = None) -> Optional[Path]:
    """Find the root directory of a RapidKit project.

    Reduced returns to satisfy lint; keeps logic readable.
    """
    result: Optional[Path] = None
    if target_project:
        base = Path.cwd()
        if target_project == "boilerplates" and (base / "boilerplates").exists():
            parent_dir = base / "boilerplates"
            choices = [p for p in sorted(parent_dir.iterdir()) if p.is_dir()]
            result = choices[0].resolve() if choices else None
        else:
            candidate = base / "boilerplates" / target_project
            if candidate.exists() and candidate.is_dir():
                if candidate.name == "boilerplates":
                    children = [p for p in sorted(candidate.iterdir()) if p.is_dir()]
                    result = children[0].resolve() if children else None
                else:
                    result = candidate.resolve()
    else:
        current = Path.cwd()
        fallback: Optional[Path] = None
        for parent in [current] + list(current.parents):
            strong, weak = _classify_project_root(parent)
            if strong:
                result = parent
                break
            if weak and fallback is None:
                fallback = parent
        if result is None and fallback is not None:
            result = fallback
    return result


def _classify_project_root(path: Path) -> tuple[bool, bool]:
    """Return (has_strong_marker, has_any_marker) for candidate project roots."""

    if not path.exists() or not path.is_dir():
        return (False, False)

    strong_markers = [".rapidkit"]
    weak_markers = [
        "pyproject.toml",
        ".env",
        "template.yaml",
        "registry.json",
        "snippet_registry.json",
    ]

    has_strong = any((path / marker).exists() for marker in strong_markers)
    if has_strong:
        return (True, True)

    has_weak = any((path / marker).exists() for marker in weak_markers)
    return (False, has_weak)


def _is_project_root(path: Path) -> bool:
    """Legacy helper retained for callers that only need a yes/no answer."""

    _, has_marker = _classify_project_root(path)
    return has_marker


PathLike = Union[str, Path]


def resolve_project_path(
    project_root: Path, root_path: Optional[PathLike], relative_path: PathLike
) -> Path:
    """Resolve a project-relative path without duplicating root prefixes."""

    rel = Path(relative_path)
    if rel.is_absolute():
        return rel

    normalized_root = Path(root_path) if root_path else Path(".")
    if str(normalized_root) in {"", "."}:
        return project_root / rel

    try:
        rel.relative_to(normalized_root)
    except ValueError:
        return project_root / normalized_root / rel
    return project_root / rel



"""Template utilities split across module scaffolding helpers."""

from .documentation import build_documentation_files
from .module_files import build_module_files
from .repository_tests import (
    repository_generator_test_template,
    repository_integration_test_template,
    repository_tests_conftest_template,
    repository_tests_init_template,
    repository_unit_test_template,
)

__all__ = [
    "build_documentation_files",
    "build_module_files",
    "repository_generator_test_template",
    "repository_integration_test_template",
    "repository_tests_conftest_template",
    "repository_tests_init_template",
    "repository_unit_test_template",
]



"""Template factories for module-level scaffold outputs."""

from __future__ import annotations

import textwrap
from pathlib import Path
from string import Template
from typing import Dict, Mapping


def build_module_files(identifiers: Mapping[str, str]) -> Dict[Path, Template]:
    """Return the mapping of module-relative file paths to rendered templates."""

    module_name = identifiers["module_name"]

    files: Dict[Path, Template] = {
        Path("README.md"): Template(_readme_template()),
        Path("__init__.py"): Template(_init_template()),
        Path("generate.py"): Template(_generator_template()),
        Path("module.yaml"): Template(_module_yaml_template()),
        Path("module.verify.json"): Template(_module_verify_template()),
        Path(".module_state.json"): Template(_module_state_template()),
        Path("overrides.py"): Template(_overrides_template()),
        Path("config/base.yaml"): Template(_base_config_template()),
        Path("config/snippets.yaml"): Template(_snippets_config_template()),
        Path("scripts/run_demo.py"): Template(_run_demo_script_template()),
    }

    files.update(
        {
            Path("frameworks/__init__.py"): Template(_frameworks_init_template()),
            Path("frameworks/fastapi.py"): Template(_framework_fastapi_template()),
            Path("frameworks/nestjs.py"): Template(_framework_nestjs_template()),
        }
    )

    files.update(_module_docs_templates())
    files.update(_module_template_files(module_name))

    return files


def _module_docs_templates() -> Dict[Path, Template]:
    return {
        Path("docs/README.md"): Template(_module_docs_readme_template()),
        Path("docs/overview.md"): Template(_module_docs_overview_template()),
        Path("docs/usage.md"): Template(_module_docs_usage_template()),
        Path("docs/monitoring.md"): Template(_module_docs_monitoring_template()),
        Path("docs/advanced.md"): Template(_module_docs_advanced_template()),
        Path("docs/changelog.md"): Template(_module_docs_changelog_template()),
        Path("docs/migration.md"): Template(_module_docs_migration_template()),
        Path("docs/troubleshooting.md"): Template(_module_docs_troubleshooting_template()),
        Path("docs/api-reference.md"): Template(_module_docs_api_reference_template()),
    }


def _module_template_files(module_name: str) -> Dict[Path, Template]:
    return {
        Path("templates/base") / f"{module_name}.py.j2": Template(_base_python_template()),
        Path("templates/base") / f"{module_name}_types.py.j2": Template(_base_types_template()),
        Path("templates/base") / f"{module_name}_health.py.j2": Template(_base_health_template()),
        Path("templates/variants/fastapi")
        / f"{module_name}.py.j2": Template(_fastapi_runtime_template()),
        Path("templates/variants/fastapi")
        / f"{module_name}_routes.py.j2": Template(_fastapi_router_template()),
        Path("templates/variants/fastapi")
        / f"{module_name}_health.py.j2": Template(_fastapi_health_template()),
        Path("templates/variants/nestjs")
        / f"{module_name}.service.ts.j2": Template(_nestjs_service_template()),
        Path("templates/variants/nestjs")
        / f"{module_name}.controller.ts.j2": Template(_nestjs_controller_template()),
        Path("templates/variants/nestjs")
        / f"{module_name}.module.ts.j2": Template(_nestjs_module_template()),
        Path("templates/variants/nestjs")
        / f"{module_name}.health.ts.j2": Template(_nestjs_health_template()),
        Path("templates/variants/nestjs")
        / f"{module_name}.validation.ts.j2": Template(_nestjs_validation_template()),
        Path("templates/variants/nestjs")
        / f"{module_name}.index.ts.j2": Template(_nestjs_index_template()),
        Path("templates/variants/nestjs")
        / f"{module_name}.configuration.ts.j2": Template(_nestjs_configuration_template()),
        Path("templates/variants/fastapi/tests")
        / f"test_{module_name}_e2e.py": Template(_fastapi_e2e_test_template()),
        Path("templates/variants/nestjs/tests")
        / f"{module_name}.e2e-spec.ts": Template(_nestjs_e2e_test_template()),
        Path("templates/vendor/nestjs/configuration.js.j2"): Template(_vendor_template()),
        Path("templates/snippets") / f"{module_name}.snippet.j2": Template(_snippet_template()),
        Path("templates/tests/__init__.py"): Template(_templates_tests_package_template()),
        Path("templates/tests/integration/__init__.py"): Template(
            _templates_tests_integration_package_template()
        ),
        Path("templates/tests/integration")
        / f"test_{module_name}_integration.j2": Template(_integration_test_template()),
    }


def _run_demo_script_template() -> str:
    return textwrap.dedent(
        '''#!/usr/bin/env python3
"""Generate a small demo project for ${module_title}.

This script is intended for module developers to quickly smoke-check generator outputs.

Usage:
  python scripts/run_demo.py fastapi
  python scripts/run_demo.py nestjs
  python scripts/run_demo.py fastapi ./tmp/demo-out
"""

from __future__ import annotations

import argparse
import os
import subprocess
import sys
from pathlib import Path


def _find_project_root(start: Path) -> Path:
    for parent in (start, *start.parents):
        if (parent / "pyproject.toml").exists() and (parent / "src").exists():
            return parent
    return start.parents[len(start.parents) - 1]


def main(argv: list[str] | None = None) -> int:
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument("variant", choices=("fastapi", "nestjs"), help="Target kit")
    parser.add_argument(
        "output_dir",
        nargs="?",
        default=None,
        help="Target output directory (default: ./tmp/${module_name}-<variant>)",
    )
    args = parser.parse_args(argv)

    script_path = Path(__file__).resolve()
    module_root = script_path.parents[1]
    project_root = _find_project_root(module_root)
    modules_root = project_root / "src" / "modules"

    try:
        rel_slug = module_root.relative_to(modules_root).as_posix()
    except ValueError:
        raise SystemExit(f"Unable to resolve module slug for {module_root}")

    module_import = f"modules.{rel_slug.replace('/', '.')}"
    default_out = project_root / "tmp" / f"${module_name}-{args.variant}"
    out_dir = Path(args.output_dir).resolve() if args.output_dir else default_out

    env = os.environ.copy()
    env.setdefault("PYTHONPATH", str(project_root / "src"))

    cmd = [sys.executable, "-m", f"{module_import}.generate", args.variant, str(out_dir)]
    subprocess.run(cmd, check=True, cwd=project_root, env=env)  # nosec
    print(f"✅ Generated demo at: {out_dir}")
    return 0


if __name__ == "__main__":
    raise SystemExit(main())
'''
    ).strip()


def _fastapi_e2e_test_template() -> str:
    return textwrap.dedent(
        '''"""E2E smoke test for ${module_title} (FastAPI).

This is intentionally lightweight and designed to run without external services.
"""

from __future__ import annotations

import importlib

import pytest


def test_router_builds_without_crashing() -> None:
    fastapi = pytest.importorskip("fastapi")
    _ = fastapi

    router_rel = "${python_router_rel}"
    router_mod_path = router_rel.replace("/", ".").removesuffix(".py")
    router_module = importlib.import_module(router_mod_path)

    router = getattr(router_module, "router", None)
    if router is None:
        build_router = getattr(router_module, "build_router", None)
        create_router = getattr(router_module, "create_router", None)
        if callable(build_router):
            router = build_router()
        elif callable(create_router):
            router = create_router()
        else:
            pytest.skip("Module router does not expose router/build_router/create_router")

    assert getattr(router, "routes", None) is not None
'''
    ).strip()


def _nestjs_e2e_test_template() -> str:
    return textwrap.dedent(
        """import { Test, TestingModule } from "@nestjs/testing";

import { {{ module_class_name }}Module } from "../../../../../src/${module_kebab}/${module_name}.module";

describe("{{ module_class_name }} NestJS E2E", () => {
  it("compiles the module", async () => {
    let moduleRef: TestingModule;
    try {
      moduleRef = await Test.createTestingModule({
        imports: [{{ module_class_name }}Module],
      }).compile();
    } catch (err) {
      // If a downstream app doesn't install optional dependencies,
      // treat this as a skipped smoke rather than failing CI.
      // eslint-disable-next-line no-console
      console.warn("Skipping module compile smoke:", err);
      return;
    }

    expect(moduleRef).toBeDefined();
  });
});
"""
    ).strip()


def _generator_template() -> str:
    return textwrap.dedent(
        '''#!/usr/bin/env python3
"""Unified module generator for ${module_title}."""

from __future__ import annotations

import sys
from pathlib import Path
from traceback import TracebackException
from typing import Any, Dict, Mapping, Optional

import yaml

from ${module_frameworks_import} import get_plugin, list_available_plugins
from modules.shared.exceptions import ModuleGeneratorError
from modules.shared.generator import TemplateRenderer, format_missing_dependencies
from modules.shared.generator.module_generator import BaseModuleGenerator
from modules.shared.versioning import ensure_version_consistency

MODULE_ROOT = Path(__file__).parent
PROJECT_ROOT = BaseModuleGenerator.detect_project_root(MODULE_ROOT)

MODULE_NAME = "${module_name}"
MODULE_CLASS = "${module_class}"
MODULE_TITLE = "${module_title}"
MODULE_TIER = "${tier}"
MODULE_SLUG = "${module_slug}"
PYTHON_OUTPUT_REL = "${python_output_rel}"
PYTHON_TYPES_REL = "${python_types_rel}"
PYTHON_HEALTH_REL = "${python_health_rel}"
NEST_OUTPUT_REL = "${nest_output_rel}"
NEST_CONTROLLER_REL = "${nest_controller_rel}"
NEST_MODULE_REL = "${nest_module_rel}"
NEST_HEALTH_REL = "${nest_health_rel}"
NEST_VALIDATION_REL = "${nest_validation_rel}"
NEST_INDEX_REL = "${nest_index_rel}"
NEST_CONFIGURATION_REL = "${nest_configuration_rel}"
VENDOR_RELATIVE = "${vendor_relative}"
VENDOR_TYPES_RELATIVE = "${vendor_types_relative}"
VENDOR_HEALTH_RELATIVE = "${vendor_health_relative}"
MODULE_IMPORT_PATH = "${module_package_import}"


class GeneratorError(ModuleGeneratorError):
    """Explicit generator failure carrying guidance for maintainers."""

    def __init__(
        self,
        message: str,
        *,
        exit_code: int = 1,
        context: Optional[Dict[str, Any]] = None,
    ) -> None:
        error_context = dict(context or {})
        error_context.setdefault("exit_code", exit_code)
        super().__init__(message, context=error_context)
        self.exit_code = exit_code


def infer_vendor_primary_path(config: Mapping[str, Any]) -> str:
    """Return the vendor relative path used as the default output."""

    vendor_cfg = config.get("generation", {}).get("vendor", {})
    for entry in vendor_cfg.get("files", []):
        template_name = entry.get("template", "")
        relative = entry.get("relative")
        if (
            isinstance(template_name, str)
            and template_name.endswith(f"{MODULE_NAME}.py.j2")
            and isinstance(relative, str)
        ):
            return relative
    return VENDOR_RELATIVE


class ${module_class}ModuleGenerator(BaseModuleGenerator):
    def __init__(self) -> None:
        super().__init__(
            module_root=MODULE_ROOT,
            templates_root=MODULE_ROOT,
            project_root=PROJECT_ROOT,
            module_identifier=MODULE_NAME,
            get_plugin=get_plugin,
            list_plugins=list_available_plugins,
            error_cls=GeneratorError,
        )

    def build_base_context(self, config: Mapping[str, Any]) -> Dict[str, Any]:
        module = str(config.get("name", MODULE_NAME))
        return {
            "module_name": module,
            "module_class_name": MODULE_CLASS,
            "module_title": MODULE_TITLE,
            "module_tier": MODULE_TIER,
            "module_slug": MODULE_SLUG,
            "module_kebab": "${module_kebab}",
            "rapidkit_vendor_module": module,
            "rapidkit_vendor_version": config.get("version", "0.1.0"),
            "rapidkit_vendor_relative_path": infer_vendor_primary_path(config),
            "rapidkit_vendor_types_path": VENDOR_TYPES_RELATIVE,
            "rapidkit_vendor_health_path": VENDOR_HEALTH_RELATIVE,
            "python_output_relative": PYTHON_OUTPUT_REL,
            "python_types_relative": PYTHON_TYPES_REL,
            "python_health_relative": PYTHON_HEALTH_REL,
            "nest_output_relative": NEST_OUTPUT_REL,
            "nest_controller_relative": NEST_CONTROLLER_REL,
            "nest_module_relative": NEST_MODULE_REL,
            "nest_health_relative": NEST_HEALTH_REL,
            "nest_validation_relative": NEST_VALIDATION_REL,
            "nest_index_relative": NEST_INDEX_REL,
            "nest_configuration_relative": NEST_CONFIGURATION_REL,
            "enabled_features": [
                "runtime-facade",
                "framework-plugins",
                "module-overrides",
                "health-checks",
            ],
        }


def _create_generator() -> ${module_class}ModuleGenerator:
    return ${module_class}ModuleGenerator()


def load_module_config() -> Dict[str, Any]:
    return dict(_create_generator().load_module_config())


def build_base_context(config: Mapping[str, Any]) -> Dict[str, Any]:
    return dict(_create_generator().build_base_context(config))


def generate_vendor_files(
    config: Mapping[str, Any],
    target_dir: Path,
    renderer: TemplateRenderer,
    context: Mapping[str, Any],
) -> None:
    _create_generator().generate_vendor_files(config, target_dir, renderer, context)


def generate_variant_files(
    variant_name: str,
    target_dir: Path,
    renderer: TemplateRenderer,
    context: Mapping[str, Any],
) -> None:
    _create_generator().generate_variant_files(variant_name, target_dir, renderer, context)


def main() -> None:
    expected_arg_count = 3
    if len(sys.argv) != expected_arg_count:
        available_plugins = list_available_plugins()
        available_names = list(available_plugins.keys())
        guidance = (
            f"Usage: python -m {MODULE_IMPORT_PATH}.generate <framework> <target_dir>\n"
            f"Example: python -m {MODULE_IMPORT_PATH}.generate fastapi ./tmp/{MODULE_NAME}\n"
            f"Available frameworks: {', '.join(available_names)}"
        )
        raise GeneratorError(
            guidance,
            exit_code=2,
            context={
                "provided_args": sys.argv[1:],
                "expected_arg_count": expected_arg_count - 1,
            },
        )

    variant_name = sys.argv[1]
    target_dir = Path(sys.argv[2]).resolve()

    missing_optional_dependencies: Dict[str, str] = {}

    try:
        generator = ${module_class}ModuleGenerator()
        config = generator.load_module_config()
        config, version_updated = ensure_version_consistency(config, module_root=MODULE_ROOT)
        base_context = generator.build_base_context(config)
        if version_updated:
            print(f"Auto bumped {MODULE_NAME} module version to {config['version']}")
        renderer = generator.create_renderer()

        generator.generate_vendor_files(config, target_dir, renderer, base_context)
        generator.generate_variant_files(variant_name, target_dir, renderer, base_context)
    except GeneratorError as exc:
        print(f"❌ Generator Error: {exc.message}")
        if exc.context:
            print("Context:")
            for key, value in exc.context.items():
                print(f"  {key}: {value}")
        dep_hint = format_missing_dependencies(missing_optional_dependencies)
        if dep_hint:
            print(f"\n{dep_hint}")
        sys.exit(exc.exit_code)
    except (RuntimeError, OSError, yaml.YAMLError) as exc:
        print("❌ Generator failed with an unexpected error:")
        traceback = "\n".join(TracebackException.from_exception(exc).format())
        print(traceback)
        print(
            f"💡 If this persists, run 'rapidkit modules doctor {MODULE_NAME}' or reinstall dependencies."
        )
        sys.exit(1)


if __name__ == "__main__":  # pragma: no cover - CLI entrypoint
    main()
'''
    ).strip()


def _readme_template() -> str:
    return textwrap.dedent(
        """
        # RapidKit ${module_title} Module

        The ${module_title_lower} module ${module_description}

        This README follows the shared RapidKit module format:

        1. **Overview & capabilities**
        1. **Installation commands**
        1. **Directory layout**
        1. **Generation workflow**
        1. **Runtime customisation hooks**
        1. **Security & audit notes**
        1. **Testing & release checklist**
        1. **Reference links**

        Use the same headings when documenting other modules so maintainers know what to expect.

        As a RapidKit module, `${module_name}` also follows the shared metadata/documentation standard:

        - `module.yaml` is the canonical source of truth (including the `documentation:` map).
        - Module docs live under `docs/` and should match the keys referenced from `module.yaml`.
        - The module changelog is maintained in `docs/changelog.md` and referenced both from `module.yaml`
          and this README.

        ______________________________________________________________________

        ## Module Capabilities

        - TODO: Summarise the most valuable behaviours this module delivers.
        - TODO: Highlight cross-cutting features (e.g. vendor snapshots, overrides, snippets).
        - TODO: Mention supported runtimes or frameworks.

        ______________________________________________________________________

        ## Install Commands

        ```bash
        rapidkit add module ${module_name}
        ```

        Re-run `rapidkit modules lock --overwrite` after adding or upgrading the module so downstream
        projects capture the new snapshot.

        ### Quickstart

        Follow the end-to-end walkthrough in `docs/usage.md`.

        ______________________________________________________________________

        ## Directory Layout

        | Path                     | Responsibility                                                       |
        | ------------------------ | -------------------------------------------------------------------- |
        | `module.yaml`            | Canonical metadata (version, compatibility, testing roster)         |
        | `config/base.yaml`       | Declarative inputs that drive prompts and dependency resolution     |
        | `generate.py`            | CLI entry point orchestrating vendor + variant generation           |
        | `frameworks/`            | Framework plugin implementations registered via `modules.shared.frameworks` |
        | `overrides.py`           | Runtime override contracts toggled via environment variables        |
        | `docs/`                  | Module docs referenced from `module.yaml` (usage/overview/changelog) |
        | `templates/`             | Base templates plus per-framework variants                          |
        | `tests/` or `tests/modules` | Generator and integration test suites                               |

        Add or prune rows to match the concrete files once the module is implemented.

        ______________________________________________________________________

        ## Generation Workflow

        1. `generate.py` loads `module.yaml`, validates schema, and checks for version drift using `modules.shared.versioning.ensure_version_consistency`.
        2. Vendor artefacts render into `.rapidkit/vendor/${module_name}/<version>` to keep upgrades auditable.
        3. Framework plugins provide template mappings, output paths, and lifecycle hooks for each variant.
        4. Optional snippets are injected via `modules.shared.generator.snippets` when enabled.
        5. Overrides may tweak the render context or perform post-processing on generated files.

        Replace the bullets once the module logic is finalised.

        ______________________________________________________________________

        ## Runtime Customisation

        Document the environment variables or override contracts exposed by `overrides.py`. Include a table similar to:

        | Environment Variable | Effect |
        | -------------------- | ------ |
        | `RAPIDKIT_${module_name_upper}_EXAMPLE` | TODO: Describe behaviour |

        Remove or expand entries to match the real override hooks.

        ______________________________________________________________________

        ## Security & Audit

        This module ships as part of the RapidKit module ecosystem and is intended to be **audited** as a unit:

        - Use `scripts/modules_doctor.py` (or `rapidkit modules vet`) to validate structure and generator invariants.
        - Use `rapidkit modules verify-all` to verify recorded hashes/signatures when running in release mode.

        If you extend this module, keep the documentation updated with the security assumptions and any threat model
        relevant to your deployment.

        ______________________________________________________________________

        ## Testing Checklist

        ```bash
        # Unit + integration suite
        poetry run pytest tests/modules/${module_slug_test_path} -q

        # Generator smoke tests
        poetry run python -m ${module_import_path}.generate fastapi ./tmp/${module_name}
        poetry run python -m ${module_import_path}.generate nestjs ./tmp/${module_name}-nestjs

        # Optional integrity audit
        poetry run python scripts/check_module_integrity.py --module ${module_slug}
        ```

        Update the commands and file paths to mirror the real test layout before publishing.

        ______________________________________________________________________

        ## Release Checklist

        1. Update templates and/or `module.yaml`.
        1. Regenerate vendor snapshots and project variants for every supported framework.
        1. Inspect rendered files (`.rapidkit/vendor` and sample project outputs) for accuracy.
        1. Execute the testing checklist above; ensure versioning is bumped when content hashes change.
        1. Commit regenerated assets alongside the updated metadata.

        ______________________________________________________________________

        ## Reference Documentation

        - Overview: `docs/overview.md`
        - Usage guide: `docs/usage.md`
        - Advanced scenarios: `docs/advanced.md`
        - Monitoring: `docs/monitoring.md`
        - Changelog: `docs/changelog.md`
        - Migration playbook: `docs/migration.md`
        - Troubleshooting: `docs/troubleshooting.md`
        - API reference: `docs/api-reference.md`
        - Override contracts: `overrides.py`

        For additional help, open an issue at <https://github.com/getrapidkit/rapidkit-core/issues> or consult the full product documentation at <https://docs.rapidkit.top>.
        """
    ).strip()


def _module_docs_readme_template() -> str:
    return textwrap.dedent(
        """
        # ${module_title} Module Documentation

        This directory contains the canonical documentation set for the ${module_title_lower} module.
        Keep guidance concise, actionable, and synchronised with the generator outputs.

        | Document | Purpose |
        | -------- | ------- |
        | [Overview](overview.md) | High-level capabilities and architecture |
        | [Usage](usage.md) | Configuration examples and integration walkthroughs |
        | [Monitoring](monitoring.md) | Metrics, telemetry, health probes, and dashboards |
        | [Advanced](advanced.md) | Extensibility patterns and override contracts |
        | [Changelog](changelog.md) | Release notes and upgrade history |
        | [Migration](migration.md) | Upgrade guidance and compatibility notes |
        | [Troubleshooting](troubleshooting.md) | Common diagnostics and remediation steps |
        | [API Reference](api-reference.md) | Public classes, functions, and CLI entrypoints |
        """
    ).strip()


def _module_docs_monitoring_template() -> str:
    return textwrap.dedent(
        """
        # ${module_title} Monitoring

        This document covers metrics, telemetry, and monitoring guidance for the ${module_title_lower} module.

        ## What to monitor
        - Functional health checks (readiness/liveness)
        - Key error rates and timeouts
        - Dependency connectivity (database/cache/third-party APIs)

        ## Suggested metrics
        - Request latency and error counts
        - Queue depth / job failures (when applicable)
        - Resource utilisation correlated with load

        ## Telemetry notes
        If you emit telemetry spans/logs, ensure sensitive data is redacted and identifiers are minimised.
        """
    ).strip()


def _module_docs_overview_template() -> str:
    return textwrap.dedent(
        """
        # ${module_title} Overview

        ## Mission
        Summarise why this module exists, the business value it delivers, and the user journeys it accelerates.

        ## Capabilities
        - Runtime features shipped with the module
        - Framework variants (FastAPI, NestJS) and parity notes
        - Optional vendor integrations or adapters

        ## Architecture
        Outline the flow from generator inputs to rendered artefacts. Highlight where overrides, snippets,
        and framework plugins plug into the pipeline.
        """
    ).strip()


def _module_docs_usage_template() -> str:
    return textwrap.dedent(
        """
        # ${module_title} Usage Guide

        ## Quickstart
        ```bash
        rapidkit modules add ${module_name} --tier ${tier} --category ${category_path}
        python -m ${module_import_path}.generate fastapi ./tmp/${module_name}
        ```

        ## Configuration
        Document required configuration keys, defaults declared in `config/base.yaml`, and how snippets
        augment the base context.

        ## Framework Examples
        Describe how to integrate the generated FastAPI router and NestJS service into an application,
        including health endpoints and dependency injection touchpoints.
        """
    ).strip()


def _module_docs_advanced_template() -> str:
    return textwrap.dedent(
        """
        # ${module_title} Advanced Topics

        Capture extensibility hooks, override strategies, observability requirements, and cross-cutting concerns.
        Provide concrete code snippets or configuration fragments wherever possible.
        """
    ).strip()


def _module_docs_migration_template() -> str:
    return textwrap.dedent(
        """
        # ${module_title} Migration Guide

        Track version upgrades, breaking changes, and operator checklists required when rolling out new releases.
        Each entry should explain the impact, upgrade steps, and fallback plan.
        """
    ).strip()


def _module_docs_troubleshooting_template() -> str:
    return textwrap.dedent(
        """
        # ${module_title} Troubleshooting

        | Symptom | Diagnostic Steps | Resolution |
        | ------- | ---------------- | ---------- |
        | Example issue | `rapidkit modules doctor ${module_slug}` | Document the remediation steps |

        Update the table as production incidents surface to build a reliable operational playbook.
        """
    ).strip()


def _module_docs_api_reference_template() -> str:
    return textwrap.dedent(
        """
        # ${module_title} API Reference

        ## Runtime Surface
        Detail public classes, functions, and dataclasses exported from the runtime templates.

        ## Configuration Schema
        Enumerate configuration keys, types, defaults, and validation semantics.

        ## CLI Entrypoints
        Describe generator commands, arguments, and expected outputs for each framework.
        """
    ).strip()


def _module_docs_changelog_template() -> str:
    return textwrap.dedent(
        """
        # Changelog — ${module_slug}

        ## 0.1.0 — Initial baseline (${today_date})

        - Initial scaffold created via `rapidkit modules scaffold`
        """
    ).strip()


def _frameworks_init_template() -> str:
    return textwrap.dedent(
        '''
        """Framework plugin registry for the ${module_title} module."""

        from __future__ import annotations

        from modules.shared.frameworks import FrameworkPlugin, PluginRegistry

        from .fastapi import FastAPIPlugin
        from .nestjs import NestJSPlugin

        PLUGIN_ENTRYPOINT_GROUP = "${module_entry_point_group}"

        _REGISTRY = PluginRegistry(entry_point_group=PLUGIN_ENTRYPOINT_GROUP)
        _BUILTIN_PLUGINS = (FastAPIPlugin, NestJSPlugin)


        def register_plugin(plugin_class: type[FrameworkPlugin]) -> None:
            """Register an additional framework plugin."""

            _REGISTRY.register(plugin_class)


        def refresh_plugin_registry(*, auto_discover: bool = True) -> None:
            """Refresh registry contents and re-register built-in implementations."""

            _REGISTRY.refresh(builtins=_BUILTIN_PLUGINS, auto_discover=auto_discover)


        def get_plugin(framework_name: str) -> FrameworkPlugin:
            """Return an instantiated plugin for the framework identifier."""

            return _REGISTRY.get(framework_name)


        def list_available_plugins() -> dict[str, str]:
            """Return available plugins mapped to their display names."""

            return _REGISTRY.list_available()


        refresh_plugin_registry(auto_discover=False)


        __all__ = [
            "FrameworkPlugin",
            "register_plugin",
            "refresh_plugin_registry",
            "get_plugin",
            "list_available_plugins",
        ]
        '''
    ).strip()


def _framework_fastapi_template() -> str:
    return textwrap.dedent(
        '''
        """FastAPI plugin for ${module_title} module scaffolding."""

        from __future__ import annotations

        from pathlib import Path
        from typing import Any, Dict, List, Mapping

        from modules.shared.frameworks import FrameworkPlugin


        class FastAPIPlugin(FrameworkPlugin):
            """Provide FastAPI-specific template and output mappings."""

            @property
            def name(self) -> str:
                return "fastapi"

            @property
            def language(self) -> str:
                return "python"

            @property
            def display_name(self) -> str:
                return "FastAPI"

            def get_template_mappings(self) -> Dict[str, str]:
                return {
                    "runtime": "templates/variants/fastapi/${module_name}.py.j2",
                    "router": "templates/variants/fastapi/${module_name}_routes.py.j2",
                    "health": "templates/variants/fastapi/${module_name}_health.py.j2",
                    "integration": "templates/tests/integration/test_${module_name}_integration.j2",
                    "e2e": "templates/variants/fastapi/tests/test_${module_name}_e2e.py",
                }

            def get_output_paths(self) -> Dict[str, str]:
                return {
                    "runtime": "${python_output_rel}",
                    "router": "${python_router_rel}",
                    "health": "${python_health_rel}",
                    "integration": "${module_integration_test}",
                    "e2e": "tests/modules/e2e/${tier}/${category_path}/${module_name}/test_${module_name}_e2e.py",
                }

            def get_context_enrichments(self, base_context: Mapping[str, Any]) -> Dict[str, Any]:
                enriched = dict(base_context)
                enriched.update({
                    "framework": "fastapi",
                    "framework_display_name": "FastAPI",
                    "language": "python",
                })
                return enriched

            def validate_requirements(self) -> List[str]:
                return []

            def get_dependencies(self) -> List[str]:
                return ["fastapi>=0.110.0"]

            def get_dev_dependencies(self) -> List[str]:
                return ["pytest-asyncio>=0.23.0", "httpx>=0.27.0"]

            def pre_generation_hook(self, output_dir: Path) -> None:
                (output_dir / "src" / "routers").mkdir(parents=True, exist_ok=True)
                (output_dir / "src" / "health").mkdir(parents=True, exist_ok=True)

            def post_generation_hook(self, output_dir: Path) -> None:
                _ = output_dir
        '''
    ).strip()


def _framework_nestjs_template() -> str:
    return textwrap.dedent(
        '''
        """NestJS plugin for ${module_title} module scaffolding."""

        from __future__ import annotations

        from pathlib import Path
        from typing import Any, Dict, List, Mapping

        from modules.shared.frameworks import FrameworkPlugin


        class NestJSPlugin(FrameworkPlugin):
            """Provide NestJS-specific template and output mappings."""

            @property
            def name(self) -> str:
                return "nestjs"

            @property
            def language(self) -> str:
                return "typescript"

            @property
            def display_name(self) -> str:
                return "NestJS"

            def get_template_mappings(self) -> Dict[str, str]:
                return {
                    "service": "templates/variants/nestjs/${module_name}.service.ts.j2",
                    "controller": "templates/variants/nestjs/${module_name}.controller.ts.j2",
                    "module": "templates/variants/nestjs/${module_name}.module.ts.j2",
                    "health": "templates/variants/nestjs/${module_name}.health.ts.j2",
                    "validation": "templates/variants/nestjs/${module_name}.validation.ts.j2",
                    "index": "templates/variants/nestjs/${module_name}.index.ts.j2",
                    "configuration": "templates/variants/nestjs/${module_name}.configuration.ts.j2",
                    "e2e": "templates/variants/nestjs/tests/${module_name}.e2e-spec.ts",
                }

            def get_output_paths(self) -> Dict[str, str]:
                return {
                    "service": "${nest_service_rel}",
                    "controller": "${nest_controller_rel}",
                    "module": "${nest_module_rel}",
                    "health": "${nest_health_rel}",
                    "validation": "${nest_validation_rel}",
                    "index": "${nest_index_rel}",
                    "configuration": "${nest_configuration_rel}",
                    "e2e": "tests/modules/e2e/${category_path}/${module_name}/${module_name}.e2e-spec.ts",
                }

            def get_context_enrichments(self, base_context: Mapping[str, Any]) -> Dict[str, Any]:
                enriched = dict(base_context)
                enriched.update({
                    "framework": "nestjs",
                    "framework_display_name": "NestJS",
                    "language": "typescript",
                })
                return enriched

            def validate_requirements(self) -> List[str]:
                return []

            def get_dependencies(self) -> List[str]:
                return ["@nestjs/common>=10.0.0"]

            def get_dev_dependencies(self) -> List[str]:
                return ["@nestjs/testing>=10.0.0", "ts-jest>=29.0.0"]

            def pre_generation_hook(self, output_dir: Path) -> None:
                (output_dir / "src").mkdir(parents=True, exist_ok=True)

            def post_generation_hook(self, output_dir: Path) -> None:
                _ = output_dir
        '''
    ).strip()


def _init_template() -> str:
    return textwrap.dedent(
        '''
        """Runtime package for the ${module_title} module."""

        __all__ = ["${module_class}"]
        '''
    ).strip()


def _module_yaml_template() -> str:
    lines = (
        "name: ${module_name}",
        "display_name: ${module_display_name_literal}",
        "description: ${module_description_literal}",
        "version: 0.1.0",
        "access: ${tier}",
        "status: draft",
        "tier: ${tier}",
        "category: ${category_path}",
        "tags:",
        "${module_tags_block}",
        "generated_from_config: true",
        "profile_inherits:",
        "  fastapi.standard: fastapi",
        "  fastapi.ddd: fastapi",
        "  nestjs.standard: nestjs",
        "config_sources:",
        "  - config/base.yaml",
        "  - config/snippets.yaml",
        "generation:",
        "  vendor:",
        "    root: .rapidkit/vendor",
        "    files:",
        "      - template: templates/base/${module_name}.py.j2",
        "        relative: ${vendor_relative}",
        "      - template: templates/base/${module_name}_types.py.j2",
        "        relative: ${vendor_types_relative}",
        "      - template: templates/base/${module_name}_health.py.j2",
        "        relative: ${vendor_health_relative}",
        "      - template: templates/vendor/nestjs/configuration.js.j2",
        "        relative: nestjs/configuration.js",
        "  variants:",
        "    fastapi: &fastapi_variant",
        "      root: .",
        "      context:",
        "        framework: fastapi",
        "        module_class_name: ${module_class}",
        "      files:",
        "        - template: templates/variants/fastapi/${module_name}.py.j2",
        "          output: ${python_output_rel}",
        "        - template: templates/variants/fastapi/${module_name}_routes.py.j2",
        "          output: ${python_router_rel}",
        "        - template: templates/variants/fastapi/${module_name}_health.py.j2",
        "          output: ${python_health_rel}",
        "        - template: templates/tests/integration/test_${module_name}_integration.j2",
        "          output: ${module_integration_test}",
        "        - template: templates/variants/fastapi/tests/test_${module_name}_e2e.py",
        "          output: tests/modules/e2e/${tier}/${category_path}/${module_name}/test_${module_name}_e2e.py",
        "",
        "    # Compatibility aliases: the installer uses kit profile names.",
        "    fastapi.standard: *fastapi_variant",
        "    fastapi.ddd: *fastapi_variant",
        "",
        "    nestjs: &nestjs_variant",
        "      root: .",
        "      context:",
        "        framework: nestjs",
        "        vendor_configuration_relative: nestjs/configuration.js",
        "        module_class_name: ${module_class}",
        "        module_kebab: ${module_kebab}",
        "      files:",
        "        - template: templates/variants/nestjs/${module_name}.service.ts.j2",
        "          output: ${nest_output_rel}",
        "        - template: templates/variants/nestjs/${module_name}.controller.ts.j2",
        "          output: ${nest_controller_rel}",
        "        - template: templates/variants/nestjs/${module_name}.module.ts.j2",
        "          output: ${nest_module_rel}",
        "        - template: templates/variants/nestjs/${module_name}.health.ts.j2",
        "          output: ${nest_health_rel}",
        "        - template: templates/variants/nestjs/${module_name}.validation.ts.j2",
        "          output: ${nest_validation_rel}",
        "        - template: templates/variants/nestjs/${module_name}.index.ts.j2",
        "          output: ${nest_index_rel}",
        "        - template: templates/variants/nestjs/${module_name}.configuration.ts.j2",
        "          output: ${nest_configuration_rel}",
        "        - template: templates/variants/nestjs/tests/${module_name}.e2e-spec.ts",
        "          output: tests/modules/e2e/${category_path}/${module_name}/${module_name}.e2e-spec.ts",
        "",
        "    nestjs.standard: *nestjs_variant",
        "  snippets:",
        "    enabled: true",
        "    config: config/snippets.yaml",
        "    default: []",
        "compatibility:",
        "${module_compatibility_block}",
        "testing:",
        "  coverage_min: ${testing_coverage_min}",
        "  integration_tests: ${testing_integration_tests}",
        "  e2e_tests: ${testing_e2e_tests}",
        "${module_unit_tests_yaml}",
        "  integration_test_files:",
        "    - ${module_integration_test}",
        "  fixtures:${module_testing_fixtures_block}",
        "documentation:",
        '  changelog: "${doc_changelog_rel_module}"',
        '  readme: "README.md"',
        '  overview: "${doc_overview_rel_module}"',
        '  usage: "${doc_usage_rel_module}"',
        '  advanced: "${doc_advanced_rel_module}"',
        '  migration: "${doc_migration_rel_module}"',
        '  troubleshooting: "${doc_troubleshooting_rel_module}"',
        '  api_docs: "${doc_api_reference_rel_module}"',
        "  examples: []",
        "support:",
        "  issues: https://github.com/getrapidkit/rapidkit-core/issues",
        "  discussions: https://github.com/getrapidkit/rapidkit-core/discussions",
        "  documentation: https://docs.rapidkit.top/modules/${module_name}",
        "changelog:",
        "- version: 0.1.0",
        "  date: '${today_date}'",
        "  notes: See docs/changelog.md",
        "validation:",
        "  pre_install:",
        "    - check_python_version",
        "  post_install:",
        "    - validate_generated_code",
        "rollback:",
        "  strategy: uninstall",
        "  backup_path: .rapidkit/backups/",
        "  max_backups: 3",
        "performance:",
        "  lazy_loading: true",
        "  caching:",
        "    templates: true",
        "    ttl: 300",
        "capabilities:",
        "${module_capabilities_block}",
        "signature: null",
        "signer_id: null",
        "signature_version: null",
    )
    return "\n".join(lines).strip()


def _templates_tests_package_template() -> str:
    return textwrap.dedent(
        '''
        """Test templates package for ${module_title} module."""

        __all__ = []
        '''
    ).strip()


def _templates_tests_integration_package_template() -> str:
    return textwrap.dedent(
        '''
        """Integration test templates for ${module_title} module."""

        __all__ = []
        '''
    ).strip()


def _module_verify_template() -> str:
    return textwrap.dedent(
        """
        {
          "module": "${module_name}",
          "version": "0.1.0",
          "templates": [],
          "generated_at": "<populate via rapidkit modules verify>"
        }
        """
    ).strip()


def _module_state_template() -> str:
    return textwrap.dedent(
        """
        {
                    "hash": "",
                    "version": "0.1.0"
        }
        """
    ).strip()


def _overrides_template() -> str:
    return textwrap.dedent(
        '''
        """Override contracts for ${module_title}."""

        from core.services.override_contracts import ConfigurableOverrideMixin


        class ${module_class}Overrides(ConfigurableOverrideMixin):
            """Extend or customize generated behaviour for ${module_title}."""

            # def custom_method(self, *args, **kwargs):
            #     """Example override."""
            #     original = self.call_original("custom_method", *args, **kwargs)
            #     return original
        '''
    ).strip()


def _base_config_template() -> str:
    lines = (
        "# Module configuration baseline for ${module_title}",
        "name: ${module_name}",
        "display_name: ${module_display_name_literal}",
        "description: ${module_description_literal}",
        "root_path: src/",
        "${module_base_config_block}",
    )
    return "\n".join(lines).strip()


def _snippets_config_template() -> str:
    lines = (
        "# Register snippet bundles for ${module_title}",
        "",
        "${module_snippet_config_block}",
    )
    return "\n".join(lines).strip()


def _base_python_template() -> str:
    return textwrap.dedent(
        '''
        from dataclasses import dataclass
        from typing import Any, Dict


        @dataclass
        class {{ module_class_name }}Config:
            """Runtime configuration for {{ module_title }}."""

            enabled: bool = True
            metadata: Dict[str, Any] | None = None


        def build_default_config() -> {{ module_class_name }}Config:
            """Return default configuration payload."""

            return {{ module_class_name }}Config()
        '''
    ).strip()


def _base_types_template() -> str:
    return textwrap.dedent(
        '''
        from dataclasses import dataclass
        from typing import Any, Mapping


        @dataclass
        class {{ module_class_name }}Context:
            """Typed runtime context shared across templates."""

            metadata: Mapping[str, Any] | None = None
            enabled: bool = True
        '''
    ).strip()


def _base_health_template() -> str:
    return textwrap.dedent(
        '''
        from datetime import datetime


        def check_health() -> dict[str, str]:
            """Return a simple health payload for the runtime facade."""

            return {
                "timestamp": datetime.utcnow().isoformat(timespec="seconds"),
                "status": "ok",
            }
        '''
    ).strip()


def _fastapi_runtime_template() -> str:
    return textwrap.dedent(
        '''
        from fastapi import APIRouter


        def create_router() -> APIRouter:
            """Sample FastAPI router for {{ module_title }}."""

            router = APIRouter()

            @router.get("/health")
            def healthcheck() -> dict[str, str]:
                return {"module": "{{ module_name }}", "status": "ok"}

            return router
        '''
    ).strip()


def _fastapi_router_template() -> str:
    return textwrap.dedent(
        '''
        from fastapi import APIRouter


        def create_router() -> APIRouter:
            """Provide an application router for {{ module_title }}."""

            router = APIRouter()

            @router.get("/health", tags=["{{ module_name }}"])
            def healthcheck() -> dict[str, str]:
                return {"module": "{{ module_name }}", "status": "ok"}

            return router
        '''
    ).strip()


def _fastapi_health_template() -> str:
    return textwrap.dedent(
        '''
        from datetime import datetime


        def module_health_status() -> dict[str, str]:
            """Return a minimal health payload for FastAPI variants."""

            return {"checked_at": datetime.utcnow().isoformat(timespec="seconds")}
        '''
    ).strip()


def _nestjs_service_template() -> str:
    return textwrap.dedent(
        """
        import { Injectable } from "@nestjs/common";

        @Injectable()
        export class {{ module_class_name }}Service {
            getStatus(): Record<string, string> {
                return { module: "{{ module_name }}", status: "ok" };
            }
        }
        """
    ).strip()


def _nestjs_controller_template() -> str:
    return textwrap.dedent(
        """
        import { Controller, Get } from "@nestjs/common";
        import { {{ module_class_name }}Service } from "./{{ module_name }}.service";

        @Controller("{{ module_kebab }}")
        export class {{ module_class_name }}Controller {
            constructor(private readonly service: {{ module_class_name }}Service) {}

            @Get("health")
            getHealth(): Record<string, string> {
                return this.service.getStatus();
            }
        }
        """
    ).strip()


def _nestjs_module_template() -> str:
    return textwrap.dedent(
        """
        import { Module } from "@nestjs/common";
        import { {{ module_class_name }}Controller } from "./{{ module_name }}.controller";
        import { {{ module_class_name }}Service } from "./{{ module_name }}.service";

        @Module({
            controllers: [{{ module_class_name }}Controller],
            providers: [{{ module_class_name }}Service],
            exports: [{{ module_class_name }}Service],
        })
        export class {{ module_class_name }}Module {}
        """
    ).strip()


def _nestjs_health_template() -> str:
    return textwrap.dedent(
        """
        export interface {{ module_class_name }}Health {
            module: string;
            status: string;
            checkedAt: string;
        }

        export function runtimeHealth(): {{ module_class_name }}Health {
            return {
                module: "{{ module_name }}",
                status: "ok",
                checkedAt: new Date().toISOString(),
            };
        }
        """
    ).strip()


def _nestjs_validation_template() -> str:
    return textwrap.dedent(
        """
        import { z } from "zod";

        export const {{ module_name }}ConfigSchema = z.object({
            enabled: z.boolean().default(true),
        });

        export type {{ module_class_name }}Config = z.infer<typeof {{ module_name }}ConfigSchema>;
        """
    ).strip()


def _nestjs_index_template() -> str:
    return textwrap.dedent(
        """
        export * from "./{{ module_name }}.module";
        export * from "./{{ module_name }}.service";
        export * from "./{{ module_name }}.controller";
        export * from "./{{ module_name }}.health";
        export * from "./{{ module_name }}.validation";
        """
    ).strip()


def _nestjs_configuration_template() -> str:
    return textwrap.dedent(
        """
        export const configuration = () => ({
            {{ module_name }}: {
                enabled: true,
            },
        });
        """
    ).strip()


def _vendor_template() -> str:
    return textwrap.dedent(
        """
        "use strict";

        function loadConfiguration() {
          return {
            module: "${module_name}",
            title: "${module_title}",
            enabled: true,
          };
        }

        module.exports = {
          loadConfiguration,
        };
        """
    ).strip()


def _snippet_template() -> str:
    return textwrap.dedent(
        """
        {# Jinja snippet used to inject shared fragments into generated artefacts. #}
        {# Name this snippet when referencing from module.yaml generation entries. #}

        {% set module_name = "${module_name}" %}

        # TODO: Provide snippet content for {{ module_name }}
        """
    ).strip()


def _integration_test_template() -> str:
    return textwrap.dedent(
        '''
        from pathlib import Path


        def test_generator_entrypoint() -> None:
            """Smoky assertion ensuring generator is importable."""

            module_root = Path(__file__).resolve().parents[3]
            assert module_root.exists()
        '''
    ).strip()


__all__ = ["build_module_files"]



from pathlib import Path
from typing import Optional, Union

import yaml

from cli.utils.pathing import resolve_modules_path, resolve_src_root
from core.kit_utils import validate_license_auto
from core.services.module_path_resolver import resolve_module_directory

# from ..utils.pathing import resolve_modules_path, resolve_src_root

MODULES_PATH = resolve_modules_path()
SRC_ROOT = resolve_src_root()
KITS_PATH = SRC_ROOT / "kits"
ADDONS_PATH = SRC_ROOT / "addons"


def enforce_module_gating(
    module_name: str,
    licenses_dir: Optional[str] = None,
    modules_path: Optional[Union[str, Path]] = None,
) -> None:
    """
    Enforce license gating for a module using the new license system.
    Accepts optional licenses_dir and modules_path for testability.
    """
    modules_base = Path(modules_path) if modules_path is not None else MODULES_PATH
    module_dir = resolve_module_directory(modules_base, module_name)
    mf = module_dir / "module.yaml"
    if not mf.exists():
        raise FileNotFoundError(f"Module {module_name} not found.")
    with open(mf) as f:
        manifest = yaml.safe_load(f)
    required_tier = manifest.get("tier")
    required_features = manifest.get("features")

    # Skip license check for free tier modules
    if required_tier == "free":
        return

    try:
        validate_license_auto(
            item_type="modules",
            item_name=module_name,
            required_tier=required_tier,
            required_features=required_features,
            licenses_dir=licenses_dir,
        )
    except Exception as e:
        # Preserve original traceback using 'from e'
        raise RuntimeError(f"Module '{module_name}' is gated: {e}") from e


def enforce_kit_gating(
    kit_name: str,
    licenses_dir: Optional[str] = None,
    kits_path: Optional[Union[str, Path]] = None,
) -> None:
    """
    Enforce license gating for a kit using the new license system.
    Accepts optional licenses_dir and kits_path for testability.
    """
    kits_base = Path(kits_path) if kits_path is not None else KITS_PATH
    kit_dir = kits_base / kit_name
    manifest_path = kit_dir / "kit.yaml"
    if not manifest_path.exists():
        raise FileNotFoundError(f"Kit {kit_name} not found.")
    with open(manifest_path) as f:
        manifest = yaml.safe_load(f)
    required_tier = manifest.get("tier")
    required_features = manifest.get("features")
    try:
        validate_license_auto(
            item_type="kits",
            item_name=kit_name,
            required_tier=required_tier,
            required_features=required_features,
            licenses_dir=licenses_dir,
        )
    except Exception as e:
        # Preserve original traceback using 'from e'
        raise RuntimeError(f"Kit '{kit_name}' is gated: {e}") from e


def enforce_addon_gating(
    addon_name: str,
    licenses_dir: Optional[str] = None,
    addons_path: Optional[Union[str, Path]] = None,
) -> None:
    """
    Enforce license gating for an addon using the new license system.
    Accepts optional licenses_dir and addons_path for testability.
    """
    addons_base = Path(addons_path) if addons_path is not None else ADDONS_PATH
    addon_dir = addons_base / addon_name
    manifest_path = addon_dir / "addon.yaml"
    if not manifest_path.exists():
        raise FileNotFoundError(f"Addon {addon_name} not found.")
    with open(manifest_path) as f:
        manifest = yaml.safe_load(f)
    required_tier = manifest.get("tier")
    required_features = manifest.get("features")
    try:
        validate_license_auto(
            item_type="addons",
            item_name=addon_name,
            required_tier=required_tier,
            required_features=required_features,
            licenses_dir=licenses_dir,
        )
    except Exception as e:
        # Preserve original traceback using 'from e'
        raise RuntimeError(f"Addon '{addon_name}' is gated: {e}") from e



import difflib
import json
from contextlib import suppress
from datetime import datetime
from pathlib import Path
from typing import Any, Dict, List, Optional

import typer

from core.rendering.template_renderer import render_template
from core.services.config_loader import load_module_config
from core.services.file_hash_registry import (
    _sha256,
    load_hashes,
    record_file_hash,
    save_hashes,
    store_snapshot,
)
from core.services.module_manifest import load_manifest_or_none

from ..ui.printer import print_error, print_info, print_success, print_warning
from ..utils.classifier import classify_file_status
from ..utils.filesystem import find_project_root
from ..utils.variables_prompt import prompt_for_variables
from .diff import MODULES_PATH, _collect_module_files

merge_app = typer.Typer(help="Merge helper: reconcile local vs template changes")


@merge_app.command("module")
def merge_module(
    name: str = typer.Argument(help="Module name"),
    profile: str = typer.Option("fastapi/minimal", help="Profile chain for template resolution"),
    project: Optional[str] = typer.Option(None, help="Project name inside boilerplates"),
    strategy: str = typer.Option(
        "prompt",
        help="Conflict strategy: prompt | prefer-current | prefer-template",
    ),
    auto_apply_template_updated: bool = typer.Option(
        False,
        help="When strategy=prompt: automatically apply template for template_updated files without prompting",
    ),
    dry_run: bool = typer.Option(False, "--dry-run", help="Dry-run (no writes)"),
    json_output: bool = typer.Option(False, "--json", help="Emit JSON result"),
) -> None:
    """
    Interactive / automated merge of modified module files.

    Targets statuses: template_updated, locally_modified, manually_updated, diverged.

    Strategy rules:
    - prefer-current: keep current file (no overwrite)
    - prefer-template: overwrite with freshly rendered template (if available)
    - prompt: interactive per-file menu (show diff / apply / keep / skip)
        with --auto-apply-template-updated it auto-applies those statuses.
    """
    project_root = find_project_root(project)
    if not project_root:
        print_error("❌ Not a valid RapidKit project.")
        raise typer.Exit(code=1)

    manifest = load_manifest_or_none(MODULES_PATH, name)
    module_version = getattr(manifest, "version", "unknown") if manifest else "unknown"

    try:
        config = load_module_config(name, profile)
    except FileNotFoundError as e:
        print_error(str(e))
        raise typer.Exit(code=1) from None

    variables = prompt_for_variables(config.get("variables", {}))
    root_path = config.get("root_path", "") or ""
    registry: Dict[str, Any] = load_hashes(project_root)
    target_statuses = {
        "template_updated",
        "locally_modified",
        "manually_updated",
        "diverged",
    }

    all_files = _collect_module_files(name, profile, config)

    actions: List[Dict[str, Any]] = []
    updates_performed = 0

    def decide_action(
        rel_file: str,
        status: str,
        has_template: bool,
        has_current: bool,
        template_preview: Optional[str],
        current_preview: Optional[str],
        interactive_allowed: bool,
    ) -> str:
        nonlocal actions
        if strategy == "prefer-current":
            return "keep-current"
        if strategy == "prefer-template":
            return "apply-template" if has_template else "skip"
        # prompt strategy
        if auto_apply_template_updated and status == "template_updated" and has_template:
            return "apply-template"

        if not interactive_allowed:
            return "apply-template" if has_template else "keep-current"

        choices = {
            "k": "keep-current",
            "c": "keep-current",
            "a": "apply-template",
            "t": "apply-template",
            "s": "skip",
        }
        default_choice = "a" if has_template else "k"
        while True:
            prompt_text = (
                f"File: {rel_file} [{status}] -> (k)eep current"
                + ("/(a)pply template" if has_template else "")
                + "/(s)kip"
                + (
                    "/(d)iff"
                    if has_template and has_current and template_preview is not None
                    else ""
                )
            )
            answer = typer.prompt(prompt_text, default=default_choice).strip().lower()
            if answer in {"d", "diff"} and has_template and has_current:
                current_lines = (current_preview or "").splitlines()
                template_lines = (template_preview or "").splitlines()
                diff_lines = difflib.unified_diff(
                    current_lines,
                    template_lines,
                    fromfile=f"{rel_file} (current)",
                    tofile=f"{rel_file} (template)",
                    lineterm="",
                    n=3,
                )
                print_info("\n".join(diff_lines) or "(no diff)")
                continue
            decision = choices.get(answer)
            if decision:
                return decision
            print_warning("Please choose one of: k, a, s" + (", d" if has_template else ""))

    for context, file_entry in all_files:
        rel = (
            (file_entry.get("path") or file_entry.get("template"))
            if isinstance(file_entry, dict)
            else file_entry
        )
        if not rel:
            continue

        template_file = file_entry.get("template") if isinstance(file_entry, dict) else None
        if template_file:
            template_path = MODULES_PATH / name / template_file
        else:
            template_path = (
                MODULES_PATH
                / name
                / "templates"
                / ("base" if context == "base" else f"overrides/{context}")
                / f"{rel}.j2"
            )
            if not template_path.exists():
                overrides_dir = MODULES_PATH / name / "templates" / "overrides"
                if overrides_dir.exists():
                    target_suffix = f"{rel}.j2"
                    for cand in overrides_dir.glob("**/*.j2"):
                        try:
                            if cand.as_posix().endswith(target_suffix):
                                template_path = cand
                                break
                        except AttributeError:
                            continue

        destination = Path(project_root / root_path / rel).resolve()
        try:
            rel_record = destination.relative_to(project_root).as_posix()
        except ValueError:
            # Destination outside project root; skip for safety
            print_warning(f"Skipping {rel}: resolves outside project root")
            continue

        entry = registry.get("files", {}).get(rel_record, {})
        registry_hash = entry.get("hash") if isinstance(entry, dict) else None

        current_bytes: Optional[bytes]
        current_hash: Optional[str]
        if destination.exists():
            try:
                current_bytes = destination.read_bytes()
            except OSError as err:
                print_warning(f"Cannot read {rel_record}: {err}")
                current_bytes = None
            current_hash = _sha256(current_bytes) if current_bytes is not None else None
        else:
            current_bytes = None
            current_hash = None

        template_content: Optional[str] = None
        template_hash: Optional[str] = None
        if template_path.exists():
            try:
                template_content = render_template(template_path, variables)
                template_hash = _sha256(template_content.encode("utf-8"))
            except (OSError, ValueError) as err:
                print_warning(f"Failed to render template for {rel_record}: {err}")

        status = classify_file_status(
            destination.exists(),
            registry_hash,
            current_hash,
            template_hash,
        )

        if status not in target_statuses:
            continue

        has_template = template_content is not None
        has_current = current_bytes is not None
        decision = decide_action(
            rel_record,
            status,
            has_template,
            has_current,
            template_content,
            current_bytes.decode("utf-8", errors="ignore") if current_bytes else None,
            strategy == "prompt" and not json_output,
        )
        actions.append(
            {
                "file": rel_record,
                "status": status,
                "decision": decision,
            }
        )

        if dry_run or decision == "skip":
            actions[-1]["applied"] = False
            continue

        if decision == "apply-template":
            if not has_template:
                actions[-1]["applied"] = False
                actions[-1]["reason"] = "template-missing"
                continue
            previous_content = current_bytes
            if previous_content is not None:
                with suppress(OSError):
                    store_snapshot(project_root, previous_content)
            destination.parent.mkdir(parents=True, exist_ok=True)
            try:
                destination.write_text(template_content or "", encoding="utf-8")
            except OSError as err:
                print_error(f"Failed writing {rel_record}: {err}")
                actions[-1]["applied"] = False
                actions[-1]["reason"] = "write-error"
                continue
            record_file_hash(
                registry,
                rel_record,
                name,
                module_version,
                (template_content or "").encode("utf-8"),
                previous_hash=registry_hash,
                snapshot=True,
                project_root=project_root,
            )
            updates_performed += 1
            actions[-1]["applied"] = True
            actions[-1]["result_hash"] = template_hash
            continue

        if decision == "keep-current":
            if current_bytes is None:
                actions[-1]["applied"] = False
                actions[-1]["reason"] = "current-missing"
                continue
            record_file_hash(
                registry,
                rel_record,
                name,
                module_version,
                current_bytes,
                previous_hash=registry_hash,
                snapshot=True,
                project_root=project_root,
            )
            updates_performed += 1
            actions[-1]["applied"] = True
            actions[-1]["result_hash"] = current_hash
            continue

        actions[-1]["applied"] = False

    if not dry_run and updates_performed:
        save_hashes(project_root, registry)

    summary = {
        "schema_version": "merge-module-v1",
        "generated_at": datetime.utcnow().isoformat() + "Z",
        "module": name,
        "profile": profile,
        "dry_run": bool(dry_run),
        "actions": actions,
    }

    if json_output:
        print(json.dumps(summary, indent=2))
        return

    applied = len([a for a in actions if a.get("applied")])
    skipped = len(actions) - applied
    print_info(f"Merge candidates processed: {len(actions)}")
    print_info(f"Applied updates: {applied} | Skipped: {skipped}")
    if updates_performed:
        print_success("Registry updated.")
    else:
        print_warning("No changes applied; registry unchanged.")



from __future__ import annotations

from importlib import import_module, metadata
from pathlib import Path
from typing import Iterable, Optional

from packaging import version as packaging_version

CURRENT_VERSION = "0.3.1"


def check_min_version(min_version: str, current_version: str) -> None:
    if min_version and packaging_version.parse(current_version) < packaging_version.parse(
        min_version
    ):
        raise RuntimeError(
            f"RapidKit version {current_version} is lower than required {min_version}"
        )


def _resolve_metadata_version(names: Iterable[str]) -> Optional[str]:
    for name in names:
        try:
            resolved = metadata.version(name)
        except metadata.PackageNotFoundError:
            continue
        except ValueError:  # pragma: no cover - defensive guard
            continue
        else:
            if resolved:
                return resolved
    return None


def _read_module_version(*module_names: str) -> Optional[str]:
    for module_name in module_names:
        try:
            module = import_module(module_name)
        except ImportError:
            continue

        candidate = getattr(module, "__version__", None)
        if isinstance(candidate, str) and candidate:
            return candidate
    return None


def _read_pyproject_version() -> Optional[str]:
    project_root = Path(__file__).resolve().parents[3]
    pyproject = project_root / "pyproject.toml"
    if not pyproject.exists():
        return None

    try:
        text = pyproject.read_text(encoding="utf-8")
    except OSError:  # pragma: no cover - I/O failure fallback
        return None

    active_section: Optional[str] = None
    for raw_line in text.splitlines():
        line = raw_line.strip()

        if not line or line.startswith("#"):
            continue

        if line.startswith("[") and line.endswith("]"):
            section_name = line.strip("[]").strip().lower()
            active_section = section_name if section_name in {"tool.poetry", "project"} else None
            continue

        if active_section in {"tool.poetry", "project"} and line.startswith("version"):
            _, _, remainder = line.partition("=")
            candidate = remainder.strip().strip("\"'")
            if candidate:
                return candidate

    return None


def get_version() -> str:
    """Return the installed RapidKit package version if available, else fall back."""
    resolved = _resolve_metadata_version(["rapidkit-core", "rapidkit_core", "rapidkit"])
    if resolved:
        return resolved

    module_version = _read_module_version("rapidkit", "core", "src")
    if module_version:
        return module_version

    pyproject_version = _read_pyproject_version()
    if pyproject_version:
        return pyproject_version

    return CURRENT_VERSION


def get_cli_version() -> str:
    """Return the RapidKit global CLI version if available."""
    resolved = _resolve_metadata_version(["rapidkit-cli", "rapidkit_cli", "rapidkit"])
    if resolved:
        return resolved

    module_version = _read_module_version("cli", "rapidkit")
    if module_version:
        return module_version

    pyproject_version = _read_pyproject_version()
    if pyproject_version:
        return pyproject_version

    return CURRENT_VERSION



"""End-to-end smoke test for AI Assistant module in FastAPI projects.

Validates that the generated routes respond correctly and basic request/response
flows work as expected.
"""

from http import HTTPStatus

import pytest


@pytest.mark.asyncio
async def test_health_endpoint(client):
    """Verify health endpoint returns ok status."""
    response = await client.get("/api/health")
    if response.status_code != HTTPStatus.OK:
        pytest.fail(f"unexpected status code: {response.status_code}")
    data = response.json()
    if "status" not in data:
        pytest.fail("missing 'status' in health response")


@pytest.mark.asyncio
async def test_health_probe_metadata(client):
    """Verify health probe includes expected metadata fields."""
    response = await client.get("/api/health")
    if response.status_code != HTTPStatus.OK:
        pytest.fail(f"unexpected status code: {response.status_code}")
    data = response.json()
    # Module health should have version and status at minimum
    if not any(k in data for k in ["version", "status", "module"]):
        pytest.fail("health response missing version/status/module keys")



"""NestJS framework plugin for the Security Headers module."""

from __future__ import annotations

from pathlib import Path
from typing import Any, Dict, Mapping

from modules.shared.frameworks import FrameworkPlugin


class NestJSPlugin(FrameworkPlugin):
    """Plugin describing NestJS-specific generation outputs."""

    @property
    def name(self) -> str:
        return "nestjs"

    @property
    def language(self) -> str:
        return "typescript"

    @property
    def display_name(self) -> str:
        return "NestJS"

    def get_template_mappings(self) -> Dict[str, str]:
        return {
            "service": "templates/variants/nestjs/security_headers.service.ts.j2",
            "controller": "templates/variants/nestjs/security_headers.controller.ts.j2",
            "module": "templates/variants/nestjs/security_headers.module.ts.j2",
            "configuration": "templates/variants/nestjs/security_headers.configuration.ts.j2",
            "health_controller": "templates/variants/nestjs/security_headers.health.controller.ts.j2",
            "health_module": "templates/variants/nestjs/security_headers.health.module.ts.j2",
            "integration_test": "templates/tests/integration/security_headers.integration.spec.ts.j2",
        }

    def get_output_paths(self) -> Dict[str, str]:
        return {
            "service": "src/modules/free/security/security_headers/security-headers/security-headers.service.ts",
            "controller": "src/modules/free/security/security_headers/security-headers/security-headers.controller.ts",
            "module": "src/modules/free/security/security_headers/security-headers/security-headers.module.ts",
            "configuration": "src/modules/free/security/security_headers/security-headers/security-headers.configuration.ts",
            "health_controller": "src/health/security-headers-health.controller.ts",
            "health_module": "src/health/security-headers-health.module.ts",
            "integration_test": "tests/modules/integration/security/security_headers.integration.spec.ts",
        }

    def get_context_enrichments(self, base_context: Mapping[str, Any]) -> Dict[str, Any]:
        return {
            **base_context,
            "framework": "nestjs",
            "framework_display_name": "NestJS",
            "module_class_name": "SecurityHeaders",
            "module_kebab": "security-headers",
            "vendor_configuration_relative": base_context.get(
                "rapidkit_vendor_configuration_relative"
            ),
            "vendor_runtime_relative": base_context.get("rapidkit_vendor_relative_path"),
            "configuration_relative": base_context.get("nest_configuration_relative"),
            "health_controller_relative": base_context.get("nest_health_controller_relative"),
            "health_module_relative": base_context.get("nest_health_module_relative"),
            "integration_test_relative": base_context.get("nest_test_relative"),
        }

    def validate_requirements(self) -> list[str]:  # noqa: D401
        """NestJS generation requires @nestjs/common to be available in the target project."""

        return []

    def get_dependencies(self) -> list[str]:
        return ["@nestjs/common", "express"]

    def pre_generation_hook(self, output_dir: Path) -> None:
        base_root = output_dir / "src" / "modules" / "free" / "security" / "security_headers"
        security_root = base_root / "security-headers"
        tests_root = output_dir / "tests" / "modules" / "integration" / "security"

        for path in (security_root, tests_root):
            path.mkdir(parents=True, exist_ok=True)

        (output_dir / "src" / "health").mkdir(parents=True, exist_ok=True)


__all__ = ["NestJSPlugin"]



"""Override contracts for Ai Assistant."""

from core.services.override_contracts import ConfigurableOverrideMixin


class AiAssistantOverrides(ConfigurableOverrideMixin):
    """Extend or customize generated behaviour for Ai Assistant."""

    # def custom_method(self, *args, **kwargs):
    #     """Example override."""
    #     original = self.call_original("custom_method", *args, **kwargs)
    #     return original



"""FastAPI framework plugin for Users Profiles."""

from __future__ import annotations

from contextlib import suppress
from pathlib import Path
from typing import Any, Dict, List, Mapping

from modules.shared.frameworks import FrameworkPlugin
from modules.shared.utils.health import ensure_health_package


class FastAPIPlugin(FrameworkPlugin):
    """Generate FastAPI-ready Users Profiles runtime."""

    @property
    def name(self) -> str:  # noqa: D401 - short alias
        return "fastapi"

    @property
    def language(self) -> str:  # noqa: D401 - short alias
        return "python"

    @property
    def display_name(self) -> str:  # noqa: D401 - short alias
        return "FastAPI"

    def get_template_mappings(self) -> Dict[str, str]:
        return {
            "profiles_runtime_bridge": "templates/variants/fastapi/users_profiles_runtime.py.j2",
            "profiles_types_runtime": "templates/variants/fastapi/users_profiles_types_runtime.py.j2",
            "profiles_init": "templates/variants/fastapi/profiles_init.py.j2",
            "profiles_models": "templates/variants/fastapi/profiles_models.py.j2",
            "profiles_dto": "templates/variants/fastapi/profiles_dto.py.j2",
            "profiles_errors": "templates/variants/fastapi/profiles_errors.py.j2",
            "profiles_repository": "templates/variants/fastapi/profiles_repository.py.j2",
            "profiles_in_memory_repository": "templates/variants/fastapi/profiles_in_memory_repository.py.j2",
            "profiles_service": "templates/variants/fastapi/profiles_service.py.j2",
            "profiles_settings": "templates/variants/fastapi/profiles_settings.py.j2",
            "profiles_dependencies": "templates/variants/fastapi/profiles_dependencies.py.j2",
            "profiles_router": "templates/variants/fastapi/profiles_router.py.j2",
            "profiles_metadata_routes": "templates/variants/fastapi/users_profiles_routes.py.j2",
            "profiles_runtime": "templates/variants/fastapi/users_profiles.py.j2",
            "profiles_config": "templates/variants/fastapi/users_profiles_config.yaml.j2",
            "integration_test": "templates/tests/integration/test_users_profiles_integration.j2",
        }

    def get_output_paths(self) -> Dict[str, str]:
        return {
            "profiles_runtime_bridge": "src/modules/free/users/users_profiles/users_profiles.py",
            "profiles_types_runtime": "src/modules/free/users/users_profiles/users_profiles_types.py",
            "profiles_init": "src/modules/free/users/users_profiles/core/users/profiles/__init__.py",
            "profiles_models": "src/modules/free/users/users_profiles/core/users/profiles/models.py",
            "profiles_dto": "src/modules/free/users/users_profiles/core/users/profiles/dto.py",
            "profiles_errors": "src/modules/free/users/users_profiles/core/users/profiles/errors.py",
            "profiles_repository": "src/modules/free/users/users_profiles/core/users/profiles/repository.py",
            "profiles_in_memory_repository": "src/modules/free/users/users_profiles/core/users/profiles/in_memory_repository.py",
            "profiles_service": "src/modules/free/users/users_profiles/core/users/profiles/service.py",
            "profiles_settings": "src/modules/free/users/users_profiles/core/users/profiles/settings.py",
            "profiles_dependencies": "src/modules/free/users/users_profiles/core/users/profiles/dependencies.py",
            "profiles_router": "src/modules/free/users/users_profiles/core/users/profiles/router.py",
            "profiles_metadata_routes": "src/modules/free/users/users_profiles/core/users/profiles/metadata_routes.py",
            "profiles_runtime": "src/modules/free/users/users_profiles/core/users/profiles/runtime.py",
            "profiles_config": "config/users/users_profiles.yaml",
            "integration_test": "tests/modules/integration/users/test_users_profiles_integration.py",
        }

    def get_context_enrichments(self, base_context: Mapping[str, Any]) -> Dict[str, Any]:
        return {
            **base_context,
            "framework": "fastapi",
            "framework_display_name": "FastAPI",
            "language": "python",
            "vendor_runtime_relative": base_context.get("rapidkit_vendor_relative_path"),
            "vendor_health_relative": base_context.get("rapidkit_vendor_health_relative"),
            "vendor_types_relative": base_context.get("rapidkit_vendor_types_relative"),
            "config_relative": base_context.get("fastapi_config_relative"),
            "integration_test_relative": base_context.get("fastapi_test_relative"),
        }

    def validate_requirements(self) -> List[str]:
        return []

    def pre_generation_hook(self, output_dir: Path) -> None:
        base = output_dir / "src" / "modules" / "free" / "users" / "users_profiles"
        (base / "core" / "users" / "profiles").mkdir(parents=True, exist_ok=True)
        (output_dir / "config" / "users").mkdir(parents=True, exist_ok=True)
        (output_dir / "tests" / "modules" / "integration" / "users").mkdir(
            parents=True, exist_ok=True
        )

        # Best-effort: materialise vendor-backed health shim (`src/health/users_profiles.py`)
        from modules.shared.utils.health import ensure_vendor_health_shim
        from modules.shared.utils.health_specs import build_standard_health_spec

        with suppress(RuntimeError, OSError):
            spec = build_standard_health_spec(Path(__file__).resolve().parents[1])
            ensure_vendor_health_shim(output_dir, spec=spec)

        ensure_health_package(
            output_dir,
            extra_imports=[
                (
                    "src.health.users_profiles",
                    "register_users_profiles_health",
                ),
            ],
        )

        # Canonical health lives under src/health; do not emit module-local health files.

    def post_generation_hook(self, output_dir: Path) -> None:  # noqa: ARG002
        return None

    def get_documentation_urls(self) -> Dict[str, str]:
        return {
            "fastapi": "https://fastapi.tiangolo.com/",
            "pydantic": "https://docs.pydantic.dev/latest/",
        }

    def get_dependencies(self) -> List[str]:
        return [
            "fastapi>=0.111.0,<1.0",
            "pydantic>=2.12.2,<3.0",
        ]

    def get_dev_dependencies(self) -> List[str]:
        return [
            "pytest>=8.3.0",
            "pytest-asyncio>=0.25.0",
        ]



"""NestJS Standard Kit package."""

from .generator import NestJSStandardGenerator
from .hooks import post_generate, pre_generate

__all__ = [
    "NestJSStandardGenerator",
    "pre_generate",
    "post_generate",
]



"""Runtime package for the PostgreSQL Database (db_postgres) module.

This module provides production-ready PostgreSQL integration with:
- Async/sync SQLAlchemy engines
- Connection pooling with configurable settings
- Transaction management with context managers
- Health check utilities
- FastAPI dependency injection support

Example:
    >>> from modules.free.database.db_postgres import get_module_info
    >>> info = get_module_info()
    >>> print(info['version'])
    1.0.4
"""

from __future__ import annotations

from typing import Any, Dict

__version__ = "1.0.4"
__all__ = ["DatabasePostgres", "get_module_info"]


def get_module_info() -> Dict[str, Any]:
    """
    Return module metadata and status information.

    Returns:
        Dictionary containing module version, status, and capabilities
    """
    return {
        "name": "db_postgres",
        "display_name": "PostgreSQL Database",
        "version": __version__,
        "status": "active",
        "tier": "free",
        "capabilities": [
            "async_engine",
            "sync_engine",
            "connection_pooling",
            "transaction_management",
            "health_checks",
            "dependency_injection",
        ],
        "frameworks": ["fastapi", "nestjs"],
        "python_version": ">=3.10,<4.0",
        "postgresql_version": ">=12.0",
    }


class DatabasePostgres:
    """
    PostgreSQL database module runtime interface.

    This class serves as a namespace for module-level operations and provides
    access to module metadata, configuration, and utilities.
    """

    @staticmethod
    def get_version() -> str:
        """Return module version."""
        return __version__

    @staticmethod
    def get_info() -> Dict[str, Any]:
        """Return complete module information."""
        return get_module_info()



# pyright: reportMissingImports=false
"""NestJS framework plugin for the Middleware module."""

from __future__ import annotations

from pathlib import Path
from typing import Any, Dict, List, Mapping

from modules.shared.frameworks import FrameworkPlugin


class NestJSPlugin(FrameworkPlugin):
    """Plugin for generating NestJS middleware integrations."""

    @property
    def name(self) -> str:
        return "nestjs"

    @property
    def language(self) -> str:
        return "typescript"

    @property
    def display_name(self) -> str:
        return "NestJS"

    def get_template_mappings(self) -> Dict[str, str]:
        return {
            "service": "templates/variants/nestjs/middleware.service.ts.j2",
            "controller": "templates/variants/nestjs/middleware.controller.ts.j2",
            "module": "templates/variants/nestjs/middleware.module.ts.j2",
            "config": "templates/variants/nestjs/middleware.configuration.ts.j2",
            "health": "templates/vendor/nestjs/middleware.health.ts.j2",
            "integration_tests": "templates/tests/integration/middleware.integration.spec.ts.j2",
        }

    def get_output_paths(self) -> Dict[str, str]:
        return {
            "service": "src/modules/free/essentials/middleware/service.ts",
            "controller": "src/modules/free/essentials/middleware/controller.ts",
            "module": "src/modules/free/essentials/middleware/module.ts",
            "config": "src/modules/free/essentials/middleware/configuration.ts",
            "health": "src/health/middleware.health.ts",
            "integration_tests": "tests/modules/integration/essentials/middleware/middleware.integration.spec.ts",
        }

    def get_context_enrichments(self, base_context: Mapping[str, Any]) -> Dict[str, Any]:
        context = dict(base_context)
        context.update(
            framework="nestjs",
            framework_display_name="NestJS",
            language="typescript",
            runtime="node",
            module_class_name="Middleware",
        )
        slug_value = str(context.get("module_slug") or context.get("module_name") or "middleware")
        module_segment = slug_value.split("/")[-1]
        context.setdefault("module_kebab", module_segment.replace("_", "-"))
        return context

    def validate_requirements(self) -> List[str]:
        return []

    def pre_generation_hook(self, output_dir: Path) -> None:
        base = output_dir / "src" / "modules" / "free" / "essentials" / "middleware"
        base.mkdir(parents=True, exist_ok=True)
        (output_dir / "nestjs").mkdir(parents=True, exist_ok=True)
        (output_dir / "tests" / "modules" / "integration" / "essentials" / "middleware").mkdir(
            parents=True, exist_ok=True
        )

        (output_dir / "src" / "health").mkdir(parents=True, exist_ok=True)

    def post_generation_hook(self, _output_dir: Path) -> None:
        return None

    def get_documentation_urls(self) -> Dict[str, str]:
        return {
            "framework_docs": "https://nestjs.com/",
            "middleware_docs": "https://docs.nestjs.com/middleware",
        }

    def get_example_configurations(self) -> Dict[str, Any]:
        return {}

    def get_dependencies(self) -> List[str]:
        return [
            "@nestjs/common",
            "@nestjs/core",
        ]

    def get_dev_dependencies(self) -> List[str]:
        return [
            "@types/node",
        ]


class NestJSStandardPlugin(NestJSPlugin):
    """Alias plugin so nestjs.standard reuses the canonical NestJS implementation."""

    @property
    def name(self) -> str:  # noqa: D401 - alias reroutes to NestJS plugin
        return "nestjs.standard"

    @property
    def display_name(self) -> str:
        return "NestJS (standard kit)"



"""Documentation template helpers for module scaffolding."""

from __future__ import annotations

import textwrap
from pathlib import Path
from string import Template
from typing import Dict, Mapping


def build_documentation_files(identifiers: Mapping[str, str]) -> Dict[Path, Template]:
    """Return repository-relative documentation files for the new module."""

    doc_parts = identifiers["doc_parts_list"].split("/") if identifiers["doc_parts_list"] else []
    doc_root = Path("docs", "modules", *doc_parts)

    return {
        doc_root / "README.md": Template(_docs_readme_template()),
        Path(identifiers["doc_overview_path"]): Template(_docs_overview_template()),
        Path(identifiers["doc_usage_path"]): Template(_docs_usage_template()),
        Path(identifiers["doc_api_reference_path"]): Template(_docs_api_reference_template()),
        Path(identifiers["doc_advanced_path"]): Template(_docs_advanced_template()),
        Path(identifiers["doc_changelog_path"]): Template(_docs_changelog_template()),
        Path(identifiers["doc_troubleshooting_path"]): Template(_docs_troubleshooting_template()),
        Path(identifiers["doc_migration_path"]): Template(_docs_migration_template()),
    }


def _docs_readme_template() -> str:
    return textwrap.dedent(
        """
        # ${module_title} Module Docs

        This directory houses the living documentation set for the ${module_title_lower} module.
        Keep each guide lightweight, actionable, and in sync with the production implementation.

        | Document | Purpose |
        | -------- | ------- |
        | [Overview](overview.md) | Executive summary, capabilities, and architecture notes |
        | [Usage](usage.md) | CLI flows, configuration examples, and integration walkthroughs |
        | [API Reference](api-reference.md) | Public classes, functions, and environment variables |
        | [Advanced](advanced.md) | Extension patterns, override contracts, and operator playbooks |
        | [Changelog](changelog.md) | Release history, breaking changes, and operator notes |
        | [Troubleshooting](troubleshooting.md) | Common issues, diagnostics, and remediation steps |
        | [Migration](migration.md) | Upgrade checklists, compatibility notes, and rollback guidance |
        """
    ).strip()


def _docs_overview_template() -> str:
    return textwrap.dedent(
        """
        # ${module_title} Overview

        ## Purpose
        Summarise why this module exists, which user journeys it accelerates, and the measurable
        outcomes it provides to product teams.

        ## Capabilities
        - Key runtime behaviours
        - Supported frameworks (FastAPI, NestJS)
        - Optional integrations or vendor touchpoints

        ## Architecture
        Describe the high-level flow from generator inputs to rendered outputs. Highlight how
        overrides, snippets, and framework plugins slot into the pipeline.
        """
    ).strip()


def _docs_usage_template() -> str:
    return textwrap.dedent(
        """
        # ${module_title} Usage

        ## Quickstart
        ```bash
        rapidkit modules add ${module_name} --tier ${tier} --category ${category_path}
        python -m ${module_import_path}.generate fastapi ./tmp/${module_name}
        ```

        ## Configuration
        Document required config keys, defaults sourced from `config/base.yaml`, and any optional
        feature flags.

        ## Framework Examples
        Provide end-to-end examples for both FastAPI and NestJS, including health endpoints and
        dependency injection patterns.
        """
    ).strip()


def _docs_api_reference_template() -> str:
    return textwrap.dedent(
        """
        # ${module_title} API Reference

        ## Runtime Surface
        Detail the primary classes and functions emitted by the runtime templates. Include method
        signatures, expected inputs/outputs, and error semantics.

        ## Environment Variables
        | Variable | Description |
        | -------- | ----------- |
        | `RAPIDKIT_${module_name_upper}_EXAMPLE` | Placeholder flag toggling experimental behaviour |

        ## CLI Entrypoints
        Document generator commands and arguments exposed to consumers.
        """
    ).strip()


def _docs_advanced_template() -> str:
    return textwrap.dedent(
        """
        # ${module_title} Advanced Guide

        Capture extension hooks, override strategies, and observability integrations. Reference
        concrete code snippets where possible and keep guidance aligned with production practices.
        """
    ).strip()


def _docs_troubleshooting_template() -> str:
    return textwrap.dedent(
        """
        # ${module_title} Troubleshooting

        | Symptom | Diagnostic Steps | Resolution |
        | ------- | ---------------- | ---------- |
        | Example issue | `rapidkit modules doctor ${module_slug}` | Describe remediation |

        Encourage contributors to note log messages, health endpoints, and verification commands
        that speed up incident response.
        """
    ).strip()


def _docs_migration_template() -> str:
    return textwrap.dedent(
        """
        # ${module_title} Migration Guide

        Track version bumps, breaking changes, and rollback strategies. Each release should include:
        - Impacted features or templates
        - Required operator actions
        - Compatibility notes with adjacent modules
        """
    ).strip()


def _docs_changelog_template() -> str:
    return textwrap.dedent(
        """
        # ${module_title} Changelog

        ## 0.1.0 — Initial baseline
        - Initial public baseline
        - See migration.md for upgrade steps
        """
    ).strip()


__all__ = ["build_documentation_files"]



# src / core / engine / registry.py
import importlib.util
from pathlib import Path
from typing import Any, Dict, List, Type

import yaml

from core.config.kit_config import KitConfig
from core.engine.generator import BaseKitGenerator
from core.exceptions import InvalidKitError, KitNotFoundError


class KitRegistry:
    """Central registry for managing all available kits"""

    def __init__(self) -> None:
        self._kits: Dict[str, KitConfig] = {}
        self._generators: Dict[str, Type[BaseKitGenerator]] = {}
        self._shared_variables: Dict[str, Dict[str, Any]] = self._load_shared_variables()
        self._load_kits()

    def _load_kits(self) -> None:
        """Load all kits from the kits directory recursively"""
        kits_dir = Path(__file__).parent.parent.parent / "kits"

        for kit_yaml in kits_dir.rglob("kit.yaml"):
            try:
                kit_dir = kit_yaml.parent
                self._load_kit(kit_dir)
            except (OSError, ImportError, RuntimeError) as e:
                print(f"Warning: Failed to load kit at {kit_yaml}: {e}")

    def _load_kit(self, kit_dir: Path) -> None:
        kit_yaml = kit_dir / "kit.yaml"
        generator_py = kit_dir / "generator.py"

        if not kit_yaml.exists():
            raise InvalidKitError(f"kit.yaml not found in {kit_dir}")

        if not generator_py.exists():
            raise InvalidKitError(f"generator.py not found in {kit_dir}")

        with open(kit_yaml, "r", encoding="utf-8") as f:
            config_data = yaml.safe_load(f)

        if not isinstance(config_data, dict):
            raise InvalidKitError(f"Invalid kit.yaml structure in {kit_dir}")

        kit_variables = config_data.get("variables", {}) or {}
        merged_variables = self._merge_variables(kit_variables)
        config_data["variables"] = merged_variables

        config = KitConfig.from_dict(config_data)
        config.path = kit_dir

        # Load generator class
        spec = importlib.util.spec_from_file_location(f"{kit_dir.name}.generator", generator_py)
        if not spec or not spec.loader:  # defensive guard for mypy/runtime
            raise ImportError(f"Failed to load spec for generator in {kit_dir}")
        module = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(module)

        generator_class = None
        for attr_name in dir(module):
            attr = getattr(module, attr_name)
            if (
                isinstance(attr, type)
                and issubclass(attr, BaseKitGenerator)
                and attr != BaseKitGenerator
            ):
                generator_class = attr
                break

        if not generator_class:
            raise InvalidKitError(f"No generator class found in {kit_dir}/generator.py")

        main_name = config.name.lower()
        if main_name in self._kits:
            return
        self._kits[main_name] = config
        self._generators[main_name] = generator_class

        for alias in config_data.get("aliases", []):
            alias_key = alias.lower().replace(" ", ".")
            if alias_key in self._kits:
                continue
            self._kits[alias_key] = config
            self._generators[alias_key] = generator_class

    def _load_shared_variables(self) -> Dict[str, Dict[str, Any]]:
        shared_path = Path(__file__).parent.parent.parent / "kits" / "shared" / "variables.yaml"
        if not shared_path.exists():
            return {}
        try:
            raw = yaml.safe_load(shared_path.read_text(encoding="utf-8")) or {}
        except (OSError, yaml.YAMLError):
            return {}
        if not isinstance(raw, dict):
            return {}
        normalized: Dict[str, Dict[str, Any]] = {}
        for key, value in raw.items():
            if isinstance(key, str) and isinstance(value, dict):
                normalized[key] = dict(value)
        return normalized

    def _merge_variables(self, kit_specific: Dict[str, Any]) -> Dict[str, Any]:
        merged: Dict[str, Any] = {name: dict(meta) for name, meta in self._shared_variables.items()}
        for name, meta in kit_specific.items():
            if name in merged and isinstance(merged[name], dict) and isinstance(meta, dict):
                merged[name].update(meta)
            else:
                merged[name] = meta
        return merged

    def get_kit(self, kit_name: str) -> KitConfig:
        """Get kit configuration by name"""
        if kit_name not in self._kits:
            raise KitNotFoundError(f"Kit '{kit_name}' not found")
        return self._kits[kit_name]

    def get_generator(self, kit_name: str) -> BaseKitGenerator:
        """Get generator instance for a kit"""
        if kit_name not in self._generators:
            raise KitNotFoundError(f"Generator for kit '{kit_name}' not found")

        config = self.get_kit(kit_name)
        if config.path is None:
            raise KitNotFoundError(f"Kit '{kit_name}' loaded without path; invalid configuration")
        generator_cls = self._generators[kit_name]
        return generator_cls(config.path, config)

    def list_kits(self) -> List[KitConfig]:
        """List all available kits"""
        return list(self._kits.values())

    def kit_exists(self, kit_name: str) -> bool:
        """Check if a kit exists"""
        return kit_name in self._kits

    def list_kits_names(self) -> List[str]:
        return list(self._kits.keys())



"""Shared helpers reusable by multiple RapidKit scaffolding kits."""

from .settings_vendor import ensure_settings_vendor_snapshot, get_settings_vendor_metadata

__all__ = ["ensure_settings_vendor_snapshot", "get_settings_vendor_metadata"]



#!/usr/bin/env python3
"""Unified module generator for Session."""

from __future__ import annotations

import contextlib
import sys
from pathlib import Path
from traceback import TracebackException
from typing import Any, Dict, Mapping, Optional

import yaml

from modules.shared.utils.health import ensure_health_package, ensure_vendor_health_shim
from modules.shared.utils.health_specs import build_standard_health_spec
from modules.shared.versioning import ensure_version_consistency

JinjaEnvironment: Optional[Any] = None
FileSystemLoader: Optional[Any] = None
StrictUndefined: Optional[Any] = None
select_autoescape: Optional[Any] = None

with contextlib.suppress(ImportError):  # pragma: no cover - optional dependency
    from jinja2 import (
        Environment as JinjaEnvironment,
        FileSystemLoader,
        StrictUndefined,
        select_autoescape,
    )

MODULE_ROOT = Path(__file__).parent
DEFAULT_ENCODING = "utf-8"

MODULE_NAME = "session"
MODULE_CLASS = "Session"
MODULE_TITLE = "Session"
MODULE_TIER = "free"
MODULE_BASE = "src/modules/free/auth/session"
PYTHON_OUTPUT_REL = f"{MODULE_BASE}/session.py"
NEST_OUTPUT_REL = f"{MODULE_BASE}/session.service.ts"
DEFAULTS_KEY = "session_defaults"


class GeneratorError(SystemExit):
    """Explicit generator failure with helpful messaging."""

    def __init__(self, message: str, *, exit_code: int = 1) -> None:
        super().__init__(exit_code)
        self.message = message


class TemplateRenderer:
    """Render Jinja2 templates with graceful fallback."""

    def __init__(self) -> None:
        self._env: Optional[Any] = None
        if JinjaEnvironment is None or FileSystemLoader is None or StrictUndefined is None:
            return

        loader = FileSystemLoader(str(MODULE_ROOT))
        if select_autoescape is None:
            raise GeneratorError(
                f"The {MODULE_TITLE} module requires jinja2 with select_autoescape support. Install or upgrade via 'pip install --upgrade jinja2'."
            )

        self._env = JinjaEnvironment(
            loader=loader,
            autoescape=select_autoescape(
                enabled_extensions=("html", "htm", "xml"),
                default_for_string=False,
                default=False,
            ),
            keep_trailing_newline=True,
            lstrip_blocks=False,
            trim_blocks=False,
            undefined=StrictUndefined,
        )

    def render(self, template_path: Path, context: Mapping[str, Any]) -> str:
        if self._env is None:
            raise GeneratorError(
                "The Session module requires jinja2 for template rendering. Install it via 'pip install jinja2'."
            )

        template = self._env.get_template(template_path.relative_to(MODULE_ROOT).as_posix())
        rendered = template.render(**context)
        if not isinstance(rendered, str):  # pragma: no cover - Jinja should always return str
            raise GeneratorError("Template rendering must produce a string output.")
        return rendered


def _format_missing_dependencies(details: Mapping[str, str]) -> str:
    if not details:
        return ""
    lines = ["Missing optional dependencies detected:"]
    for package, hint in details.items():
        lines.append(f"  - {package}: {hint}")
    return "\n".join(lines)


def load_module_config() -> Dict[str, Any]:
    config_path = MODULE_ROOT / "module.yaml"
    with config_path.open(encoding=DEFAULT_ENCODING) as fh:
        data = yaml.safe_load(fh)
    if not isinstance(data, Mapping):
        raise ValueError("module.yaml must resolve to a mapping")
    return dict(data)


def infer_vendor_settings_path(config: Mapping[str, Any]) -> str:
    vendor_cfg = config.get("generation", {}).get("vendor", {})
    for entry in vendor_cfg.get("files", []):
        template_name = entry.get("template", "")
        relative = entry.get("relative")
        if isinstance(relative, str) and template_name.endswith(f"{MODULE_NAME}.py.j2"):
            return relative
    return PYTHON_OUTPUT_REL


def _load_defaults(config: Mapping[str, Any]) -> Dict[str, Any]:
    defaults: Any = config.get("defaults")
    if isinstance(defaults, Mapping):
        return dict(defaults)

    defaults_path = MODULE_ROOT / "config" / "base.yaml"
    if defaults_path.exists():
        with defaults_path.open(encoding=DEFAULT_ENCODING) as fh:
            base_config = yaml.safe_load(fh)
        if isinstance(base_config, Mapping):
            defaults = base_config.get("defaults")
            if isinstance(defaults, Mapping):
                return dict(defaults)
    return {}


def build_base_context(config: Mapping[str, Any]) -> Dict[str, Any]:
    module = str(config.get("name", MODULE_NAME))
    defaults = _load_defaults(config)
    context = {
        "rapidkit_vendor_module": module,
        "rapidkit_vendor_version": config.get("version", "0.0.0"),
        "rapidkit_vendor_relative_path": infer_vendor_settings_path(config),
        "module_name": module,
        "module_class_name": MODULE_CLASS,
        "module_title": MODULE_TITLE,
        "python_output_relative": PYTHON_OUTPUT_REL,
        "nest_output_relative": NEST_OUTPUT_REL,
        "module_tier": MODULE_TIER,
        "module_defaults": defaults,
        DEFAULTS_KEY: defaults,
    }
    return context


def write_file(destination: Path, content: str) -> None:
    destination.parent.mkdir(parents=True, exist_ok=True)
    destination.write_text(content, encoding=DEFAULT_ENCODING)
    print(f"Generated: {destination}")


def generate_vendor_files(
    config: Mapping[str, Any],
    target_dir: Path,
    renderer: TemplateRenderer,
    context: Mapping[str, Any],
) -> None:
    vendor_cfg = config.get("generation", {}).get("vendor")
    if not vendor_cfg:
        return

    root = vendor_cfg.get("root", ".rapidkit/vendor")
    files = vendor_cfg.get("files", [])
    module_name = context["rapidkit_vendor_module"]
    version = context["rapidkit_vendor_version"]

    for entry in files:
        template_name = entry.get("template")
        if not isinstance(template_name, str):
            raise GeneratorError("Vendor file entry missing 'template' reference in module.yaml")

        template_path = MODULE_ROOT / template_name
        if not template_path.exists():
            raise GeneratorError(
                f"Vendor template '{template_path.relative_to(MODULE_ROOT)}' missing."
            )
        relative = entry.get("relative")
        if not isinstance(relative, str):
            raise GeneratorError("Vendor file entry missing 'relative' destination in module.yaml")

        file_context = {**context, **entry.get("context", {})}
        output = target_dir / root / module_name / version / relative
        rendered = renderer.render(template_path, file_context)
        write_file(output, rendered)

    # vendor-only generation shouldn't materialise project files


def generate_variant_files(
    config: Mapping[str, Any],
    variant_name: str,
    target_dir: Path,
    renderer: TemplateRenderer,
    context: Mapping[str, Any],
) -> None:
    variants = config.get("generation", {}).get("variants", {})
    variant_cfg = variants.get(variant_name)
    if variant_cfg is None:
        available = ", ".join(sorted(variants)) or "<none>"
        raise GeneratorError(
            f"Variant '{variant_name}' not defined in module.yaml (available: {available})."
        )

    root = variant_cfg.get("root", ".")
    variant_context = variant_cfg.get("context", {})

    for entry in variant_cfg.get("files", []):
        template_name = entry.get("template")
        if not isinstance(template_name, str):
            raise GeneratorError(
                f"Variant '{variant_name}' file missing 'template' entry in module.yaml"
            )

        template_path = MODULE_ROOT / template_name
        if not template_path.exists():
            raise GeneratorError(
                f"Variant template '{template_path.relative_to(MODULE_ROOT)}' not found."
            )

        output_value = entry.get("output")
        if not isinstance(output_value, str):
            raise GeneratorError(
                f"Variant '{variant_name}' file missing 'output' destination in module.yaml"
            )

        output = Path(target_dir) / root / output_value
        file_context = {
            **context,
            **variant_context,
            **entry.get("context", {}),
        }
        rendered = renderer.render(template_path, file_context)
        write_file(output, rendered)

    # After variant generation, materialise canonical vendor-backed health shim
    # and variant canonical wrapper (if present). Prefer vendor runtime for
    # the public alias when available.
    spec = build_standard_health_spec(MODULE_ROOT)
    with contextlib.suppress(RuntimeError, OSError):
        ensure_vendor_health_shim(target_dir, spec=spec)

        variant_health_template = (
            MODULE_ROOT / "templates" / "variants" / variant_name / f"{MODULE_NAME}_health.py.j2"
        )
        canonical_path = Path(target_dir) / spec.target_relative_path
        if variant_health_template.exists():
            try:
                rendered = renderer.render(variant_health_template, {**context, **variant_context})
                canonical_path.parent.mkdir(parents=True, exist_ok=True)
                canonical_path.write_text(rendered, encoding=DEFAULT_ENCODING)
            except (OSError, RuntimeError):
                # best-effort: don't fail generation if template rendering or file writes fail
                pass

        # vendor payload (if present) is stored under .rapidkit/vendor/<module>/<version>
        # Do not create legacy alias files under canonical-only policy.
        # The canonical vendor-backed shim is at spec.target_relative_path and will be used
        # by the generated project. If a vendor payload exists it will be available under
        # the .rapidkit/vendor layout and ensure_vendor_health_shim already created the
        # canonical shim at spec.target_relative_path.
        # Do not create legacy alias files under canonical-only policy.
        # The canonical vendor-backed shim is at spec.target_relative_path and will be used
        # by the generated project. If a vendor payload exists it will be available under
        # the .rapidkit/vendor layout and ensure_vendor_health_shim already created the
        # canonical shim at spec.target_relative_path.

    ensure_health_package(
        target_dir,
        extra_imports=[(f"src.health.{spec.module_name}", f"register_{spec.module_name}_health")],
    )


def main() -> None:
    EXPECTED_ARG_COUNT = 3
    if len(sys.argv) != EXPECTED_ARG_COUNT:
        guidance = (
            "Usage: python -m modules.free.auth.session.generate <variant> <target_dir>\n"
            "Example: python -m modules.free.auth.session.generate fastapi ../../examples/session\n"
            "Variants are declared in module.yaml"
        )
        raise GeneratorError(guidance, exit_code=2)

    variant_name = sys.argv[1]
    target_dir = Path(sys.argv[2]).resolve()

    missing_optional_dependencies: Dict[str, str] = {}
    if JinjaEnvironment is None:
        missing_optional_dependencies["jinja2"] = (
            "Install to unlock advanced templating (pip install jinja2)"
        )

    try:
        config = load_module_config()
        config, version_updated = ensure_version_consistency(
            config,
            module_root=MODULE_ROOT,
        )
        base_context = build_base_context(config)
        if version_updated:
            print(f"Auto bumped module version to {config['version']}")
        renderer = TemplateRenderer()

        generate_vendor_files(config, target_dir, renderer, base_context)
        generate_variant_files(config, variant_name, target_dir, renderer, base_context)
    except GeneratorError as exc:
        message = exc.message if isinstance(exc.message, str) else str(exc.message)
        dep_hint = _format_missing_dependencies(missing_optional_dependencies)
        if dep_hint:
            message = f"{message}\n\n{dep_hint}"
        print(message)
        sys.exit(exc.args[0] if exc.args else 1)
    except (RuntimeError, OSError, yaml.YAMLError) as exc:
        print("Generator failed with an unexpected error:")
        traceback = "\n".join(TracebackException.from_exception(exc).format())
        print(traceback)
        print("If this persists, run 'rapidkit modules doctor' or reinstall dependencies.")
        sys.exit(1)


if __name__ == "__main__":
    main()
